package java.util;

import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.util.stream.IntStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.stream.StreamSupport;
import java.util.function.Function;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.util.function.UnaryOperator;
import java.util.stream.Stream;
import java.util.function.Predicate;
import java.io.Serializable;
import java.util.function.Consumer;
import java.lang.reflect.Array;

public class Collections
{
    private static final int BINARYSEARCH_THRESHOLD = 5000;
    private static final int REVERSE_THRESHOLD = 18;
    private static final int SHUFFLE_THRESHOLD = 5;
    private static final int FILL_THRESHOLD = 25;
    private static final int ROTATE_THRESHOLD = 100;
    private static final int COPY_THRESHOLD = 10;
    private static final int REPLACEALL_THRESHOLD = 11;
    private static final int INDEXOFSUBLIST_THRESHOLD = 35;
    private static Random r;
    public static final Set EMPTY_SET;
    public static final List EMPTY_LIST;
    public static final Map EMPTY_MAP;
    
    private Collections() {
    }
    
    public static <T extends Comparable<? super T>> void sort(List<T> list) {
        list.sort(null);
    }
    
    public static <T> void sort(List<T> list, Comparator<? super T> comparator) {
        list.sort(comparator);
    }
    
    public static <T> int binarySearch(List<? extends Comparable<? super T>> list, T t) {
        if (list instanceof RandomAccess || list.size() < 5000) {
            return indexedBinarySearch(list, t);
        }
        return iteratorBinarySearch(list, t);
    }
    
    private static <T> int indexedBinarySearch(List<? extends Comparable<? super T>> list, T t) {
        int i = 0;
        int n = list.size() - 1;
        while (i <= n) {
            int n2 = i + n >>> 1;
            int compareTo = ((Comparable)list.get(n2)).compareTo(t);
            if (compareTo < 0) {
                i = n2 + 1;
            }
            else if (compareTo > 0) {
                n = n2 - 1;
            }
            else {
                return n2;
            }
        }
        return -(i + 1);
    }
    
    private static <T> int iteratorBinarySearch(List<? extends Comparable<? super T>> list, T t) {
        int i = 0;
        int n = list.size() - 1;
        ListIterator<Object> listIterator = list.listIterator();
        while (i <= n) {
            int n2 = i + n >>> 1;
            int compareTo = ((Comparable)get(listIterator, n2)).compareTo(t);
            if (compareTo < 0) {
                i = n2 + 1;
            }
            else if (compareTo > 0) {
                n = n2 - 1;
            }
            else {
                return n2;
            }
        }
        return -(i + 1);
    }
    
    private static <T> T get(ListIterator<? extends T> listIterator, int n) {
        int nextIndex = listIterator.nextIndex();
        if (nextIndex <= n) {
            Object o;
            do {
                o = listIterator.next();
            } while (nextIndex++ < n);
            return o;
        }
        Object o;
        do {
            o = listIterator.previous();
        } while (--nextIndex > n);
        return o;
    }
    
    public static <T> int binarySearch(List<? extends T> list, T t, Comparator<? super T> comparator) {
        if (comparator == null) {
            return binarySearch(list, t);
        }
        if (list instanceof RandomAccess || list.size() < 5000) {
            return indexedBinarySearch(list, t, comparator);
        }
        return iteratorBinarySearch(list, t, comparator);
    }
    
    private static <T> int indexedBinarySearch(List<? extends T> list, T t, Comparator<? super T> comparator) {
        int i = 0;
        int n = list.size() - 1;
        while (i <= n) {
            int n2 = i + n >>> 1;
            int compare = comparator.compare(list.get(n2), t);
            if (compare < 0) {
                i = n2 + 1;
            }
            else if (compare > 0) {
                n = n2 - 1;
            }
            else {
                return n2;
            }
        }
        return -(i + 1);
    }
    
    private static <T> int iteratorBinarySearch(List<? extends T> list, T t, Comparator<? super T> comparator) {
        int i = 0;
        int n = list.size() - 1;
        ListIterator<Object> listIterator = list.listIterator();
        while (i <= n) {
            int n2 = i + n >>> 1;
            int compare = comparator.compare(get(listIterator, n2), t);
            if (compare < 0) {
                i = n2 + 1;
            }
            else if (compare > 0) {
                n = n2 - 1;
            }
            else {
                return n2;
            }
        }
        return -(i + 1);
    }
    
    public static void reverse(List<?> list) {
        int size = list.size();
        if (size < 18 || list instanceof RandomAccess) {
            for (int j = 0, n = size >> 1, n2 = size - 1; j < n; ++j, --n2) {
                swap(list, j, n2);
            }
        }
        else {
            ListIterator<Object> listIterator = list.listIterator();
            ListIterator<Object> listIterator2 = list.listIterator(size);
            for (int i = 0; i < list.size() >> 1; ++i) {
                Object next = listIterator.next();
                listIterator.set(listIterator2.previous());
                listIterator2.set(next);
            }
        }
    }
    
    public static void shuffle(List<?> list) {
        Random r = Collections.r;
        if (r == null) {
            r = (Collections.r = new Random());
        }
        shuffle(list, r);
    }
    
    public static void shuffle(List<?> list, Random random) {
        int i = list.size();
        if (i < 5 || list instanceof RandomAccess) {
            for (; i > 1; --i) {
                swap(list, i - 1, random.nextInt(i));
            }
        }
        else {
            Object[] array = list.toArray();
            for (; i > 1; --i) {
                swap(array, i - 1, random.nextInt(i));
            }
            ListIterator<Object> listIterator = list.listIterator();
            for (int j = 0; j < array.length; ++j) {
                listIterator.next();
                listIterator.set(array[j]);
            }
        }
    }
    
    public static void swap(List<?> list, int n, int n2) {
        list.set(n, list.set(n2, list.get(n)));
    }
    
    private static void swap(Object[] array, int n, int n2) {
        Object o = array[n];
        array[n] = array[n2];
        array[n2] = o;
    }
    
    public static <T> void fill(List<? super T> list, T t) {
        int size = list.size();
        if (size < 25 || list instanceof RandomAccess) {
            for (int j = 0; j < size; ++j) {
                list.set(j, t);
            }
        }
        else {
            ListIterator<T> listIterator = list.listIterator();
            for (int i = 0; i < size; ++i) {
                listIterator.next();
                listIterator.set(t);
            }
        }
    }
    
    public static <T> void copy(List<? super T> list, List<? extends T> list2) {
        int size = list2.size();
        if (size > list.size()) {
            throw new IndexOutOfBoundsException("Source does not fit in dest");
        }
        if (size < 10 || (list2 instanceof RandomAccess && list instanceof RandomAccess)) {
            for (int j = 0; j < size; ++j) {
                list.set(j, list2.get(j));
            }
        }
        else {
            ListIterator<Object> listIterator = list.listIterator();
            ListIterator<Object> listIterator2 = list2.listIterator();
            for (int i = 0; i < size; ++i) {
                listIterator.next();
                listIterator.set(listIterator2.next());
            }
        }
        return;
    }
    
    public static <T extends Object & Comparable<? super T>> T min(Collection<? extends T> collection) {
        Iterator<Object> iterator = collection.iterator();
        Object next = iterator.next();
        while (iterator.hasNext()) {
            Object next2 = iterator.next();
            if (((T)next2).compareTo(next) < 0) {
                next = next2;
            }
            else {
                continue;
            }
        }
        return next;
    }
    
    public static <T> T min(Collection<? extends T> collection, Comparator<? super T> comparator) {
        if (comparator == null) {
            return min(collection);
        }
        Iterator<Object> iterator = collection.iterator();
        Object next = iterator.next();
        while (iterator.hasNext()) {
            Object next2 = iterator.next();
            if (comparator.compare(next2, next) < 0) {
                next = next2;
            }
            else {
                continue;
            }
        }
        return next;
    }
    
    public static <T extends Object & Comparable<? super T>> T max(Collection<? extends T> collection) {
        Iterator<Object> iterator = collection.iterator();
        Object next = iterator.next();
        while (iterator.hasNext()) {
            Object next2 = iterator.next();
            if (((T)next2).compareTo(next) > 0) {
                next = next2;
            }
            else {
                continue;
            }
        }
        return next;
    }
    
    public static <T> T max(Collection<? extends T> collection, Comparator<? super T> comparator) {
        if (comparator == null) {
            return max(collection);
        }
        Iterator<Object> iterator = collection.iterator();
        Object next = iterator.next();
        while (iterator.hasNext()) {
            Object next2 = iterator.next();
            if (comparator.compare(next2, next) > 0) {
                next = next2;
            }
            else {
                continue;
            }
        }
        return next;
    }
    
    public static void rotate(List<?> list, int n) {
        if (list instanceof RandomAccess || list.size() < 100) {
            rotate1(list, n);
        }
        else {
            rotate2(list, n);
        }
    }
    
    private static <T> void rotate1(List<T> list, int n) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        n %= size;
        if (n < 0) {
            n += size;
        }
        if (n == 0) {
            return;
        }
        for (int i = 0, j = 0; j != size; ++i) {
            Object o = list.get(i);
            do {
                i += n;
                if (i >= size) {
                    i -= size;
                }
                o = list.set(i, o);
                ++j;
            } while (i != i);
        }
        return;
    }
    
    private static void rotate2(List<?> list, int n) {
        int size = list.size();
        if (size == 0) {
            return;
        }
        int n2 = -n % size;
        if (n2 < 0) {
            n2 += size;
        }
        if (n2 == 0) {
            return;
        }
        reverse(list.subList(0, n2));
        reverse(list.subList(n2, size));
        reverse(list);
        return;
    }
    
    public static <T> boolean replaceAll(List<T> list, T t, T t2) {
        boolean n = false;
        int size = list.size();
        if (size < 11 || list instanceof RandomAccess) {
            if (t == null) {
                for (int l = 0; l < size; ++l) {
                    if (list.get(l) == null) {
                        list.set(l, t2);
                        n = true;
                    }
                }
            }
            else {
                for (int k = 0; k < size; ++k) {
                    if (t.equals(list.get(k))) {
                        list.set(k, t2);
                        n = true;
                    }
                }
            }
        }
        else {
            ListIterator<T> listIterator = list.listIterator();
            if (t == null) {
                for (int j = 0; j < size; ++j) {
                    if (listIterator.next() == null) {
                        listIterator.set(t2);
                        n = true;
                    }
                }
            }
            else {
                for (int i = 0; i < size; ++i) {
                    if (t.equals(listIterator.next())) {
                        listIterator.set(t2);
                        n = true;
                    }
                }
            }
        }
        return n;
    }
    
    public static int indexOfSubList(List<?> list, List<?> list2) {
        int size = list.size();
        int size2 = list2.size();
        int n = size - size2;
        if (size < 35 || (list instanceof RandomAccess && list2 instanceof RandomAccess)) {
            int l = 0;
        Label_0042:
            while (l <= n) {
                int n2 = 0;
                while (n2 < size2) {
                    if (!eq(list2.get(n2), list.get(l))) {
                        ++l;
                        continue Label_0042;
                    }
                    ++n2;
                    ++l;
                }
                return l;
            }
        }
        else {
            ListIterator<Object> listIterator = list.listIterator();
            int i = 0;
        Label_0119:
            while (i <= n) {
                ListIterator<Object> listIterator2 = list2.listIterator();
                int j = 0;
                while (j < size2) {
                    if (!eq(listIterator2.next(), listIterator.next())) {
                        for (int k = 0; k < j; ++k) {
                            listIterator.previous();
                        }
                        ++i;
                        continue Label_0119;
                    }
                    ++j;
                }
                return i;
            }
        }
        return -1;
    }
    
    public static int lastIndexOfSubList(List<?> list, List<?> list2) {
        int size = list.size();
        int size2 = list2.size();
        int i = size - size2;
        if (size < 35 || list instanceof RandomAccess) {
        Label_0036:
            while (i >= 0) {
                int l = 0;
                while (l < size2) {
                    if (!eq(list2.get(l), list.get(i))) {
                        --i;
                        continue Label_0036;
                    }
                    ++l;
                    ++i;
                }
                return i;
            }
        }
        else {
            if (i < 0) {
                return -1;
            }
            ListIterator<Object> listIterator = list.listIterator(i);
        Label_0121:
            while (i >= 0) {
                ListIterator<Object> listIterator2 = list2.listIterator();
                int j = 0;
                while (j < size2) {
                    if (!eq(listIterator2.next(), listIterator.next())) {
                        if (i != 0) {
                            for (int k = 0; k <= j + 1; ++k) {
                                listIterator.previous();
                            }
                        }
                        --i;
                        continue Label_0121;
                    }
                    else {
                        ++j;
                    }
                }
                return i;
            }
        }
        return -1;
    }
    
    public static <T> Collection<T> unmodifiableCollection(Collection<? extends T> collection) {
        return new Collections.UnmodifiableCollection(collection);
    }
    
    public static <T> Set<T> unmodifiableSet(Set<? extends T> set) {
        return new Collections.UnmodifiableSet(set);
    }
    
    public static <T> SortedSet<T> unmodifiableSortedSet(SortedSet<T> set) {
        return new Collections.UnmodifiableSortedSet(set);
    }
    
    public static <T> NavigableSet<T> unmodifiableNavigableSet(NavigableSet<T> set) {
        return new Collections.UnmodifiableNavigableSet(set);
    }
    
    public static <T> List<T> unmodifiableList(List<? extends T> list) {
        return (list instanceof RandomAccess) ? new Collections.UnmodifiableRandomAccessList(list) : new Collections.UnmodifiableList(list);
    }
    
    public static <K, V> Map<K, V> unmodifiableMap(Map<? extends K, ? extends V> map) {
        return new Collections.UnmodifiableMap(map);
    }
    
    public static <K, V> SortedMap<K, V> unmodifiableSortedMap(SortedMap<K, ? extends V> sortedMap) {
        return new Collections.UnmodifiableSortedMap(sortedMap);
    }
    
    public static <K, V> NavigableMap<K, V> unmodifiableNavigableMap(NavigableMap<K, ? extends V> navigableMap) {
        return new Collections.UnmodifiableNavigableMap(navigableMap);
    }
    
    public static <T> Collection<T> synchronizedCollection(Collection<T> collection) {
        return new Collections.SynchronizedCollection(collection);
    }
    
    static <T> Collection<T> synchronizedCollection(Collection<T> collection, Object o) {
        return new Collections.SynchronizedCollection(collection, o);
    }
    
    public static <T> Set<T> synchronizedSet(Set<T> set) {
        return new Collections.SynchronizedSet(set);
    }
    
    static <T> Set<T> synchronizedSet(Set<T> set, Object o) {
        return new Collections.SynchronizedSet(set, o);
    }
    
    public static <T> SortedSet<T> synchronizedSortedSet(SortedSet<T> set) {
        return new Collections.SynchronizedSortedSet(set);
    }
    
    public static <T> NavigableSet<T> synchronizedNavigableSet(NavigableSet<T> set) {
        return new Collections.SynchronizedNavigableSet(set);
    }
    
    public static <T> List<T> synchronizedList(List<T> list) {
        return (list instanceof RandomAccess) ? new Collections.SynchronizedRandomAccessList(list) : new Collections.SynchronizedList(list);
    }
    
    static <T> List<T> synchronizedList(List<T> list, Object o) {
        return (list instanceof RandomAccess) ? new Collections.SynchronizedRandomAccessList(list, o) : new Collections.SynchronizedList(list, o);
    }
    
    public static <K, V> Map<K, V> synchronizedMap(Map<K, V> map) {
        return new Collections.SynchronizedMap(map);
    }
    
    public static <K, V> SortedMap<K, V> synchronizedSortedMap(SortedMap<K, V> sortedMap) {
        return new Collections.SynchronizedSortedMap(sortedMap);
    }
    
    public static <K, V> NavigableMap<K, V> synchronizedNavigableMap(NavigableMap<K, V> navigableMap) {
        return new Collections.SynchronizedNavigableMap(navigableMap);
    }
    
    public static <E> Collection<E> checkedCollection(Collection<E> collection, Class<E> clazz) {
        return new Collections.CheckedCollection(collection, clazz);
    }
    
    static <T> T[] zeroLengthArray(Class<T> componentType) {
        return (Object[])(Object[])Array.newInstance(componentType, 0);
    }
    
    public static <E> Queue<E> checkedQueue(Queue<E> queue, Class<E> clazz) {
        return new Collections.CheckedQueue(queue, clazz);
    }
    
    public static <E> Set<E> checkedSet(Set<E> set, Class<E> clazz) {
        return new Collections.CheckedSet(set, clazz);
    }
    
    public static <E> SortedSet<E> checkedSortedSet(SortedSet<E> set, Class<E> clazz) {
        return new Collections.CheckedSortedSet(set, clazz);
    }
    
    public static <E> NavigableSet<E> checkedNavigableSet(NavigableSet<E> set, Class<E> clazz) {
        return new Collections.CheckedNavigableSet(set, clazz);
    }
    
    public static <E> List<E> checkedList(List<E> list, Class<E> clazz) {
        return (list instanceof RandomAccess) ? new Collections.CheckedRandomAccessList(list, clazz) : new Collections.CheckedList(list, clazz);
    }
    
    public static <K, V> Map<K, V> checkedMap(Map<K, V> map, Class<K> clazz, Class<V> clazz2) {
        return new Collections.CheckedMap(map, clazz, clazz2);
    }
    
    public static <K, V> SortedMap<K, V> checkedSortedMap(SortedMap<K, V> sortedMap, Class<K> clazz, Class<V> clazz2) {
        return new Collections.CheckedSortedMap(sortedMap, clazz, clazz2);
    }
    
    public static <K, V> NavigableMap<K, V> checkedNavigableMap(NavigableMap<K, V> navigableMap, Class<K> clazz, Class<V> clazz2) {
        return new Collections.CheckedNavigableMap(navigableMap, clazz, clazz2);
    }
    
    public static <T> Iterator<T> emptyIterator() {
        return Collections.EmptyIterator.EMPTY_ITERATOR;
    }
    
    public static <T> ListIterator<T> emptyListIterator() {
        return Collections.EmptyListIterator.EMPTY_ITERATOR;
    }
    
    public static <T> Enumeration<T> emptyEnumeration() {
        return Collections.EmptyEnumeration.EMPTY_ENUMERATION;
    }
    
    public static final <T> Set<T> emptySet() {
        return Collections.EMPTY_SET;
    }
    
    public static <E> SortedSet<E> emptySortedSet() {
        return Collections.UnmodifiableNavigableSet.EMPTY_NAVIGABLE_SET;
    }
    
    public static <E> NavigableSet<E> emptyNavigableSet() {
        return Collections.UnmodifiableNavigableSet.EMPTY_NAVIGABLE_SET;
    }
    
    public static final <T> List<T> emptyList() {
        return Collections.EMPTY_LIST;
    }
    
    public static final <K, V> Map<K, V> emptyMap() {
        return Collections.EMPTY_MAP;
    }
    
    public static final <K, V> SortedMap<K, V> emptySortedMap() {
        return Collections.UnmodifiableNavigableMap.EMPTY_NAVIGABLE_MAP;
    }
    
    public static final <K, V> NavigableMap<K, V> emptyNavigableMap() {
        return Collections.UnmodifiableNavigableMap.EMPTY_NAVIGABLE_MAP;
    }
    
    public static <T> Set<T> singleton(T t) {
        return new Collections.SingletonSet(t);
    }
    
    static <E> Iterator<E> singletonIterator(final E e) {
        return new Iterator<E>() {
            private boolean hasNext = true;
            
            @Override
            public boolean hasNext() {
                return this.hasNext;
            }
            
            @Override
            public E next() {
                if (this.hasNext) {
                    this.hasNext = false;
                    return e;
                }
                throw new NoSuchElementException();
            }
            
            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
            
            @Override
            public void forEachRemaining(Consumer<? super E> consumer) {
                Objects.requireNonNull(consumer);
                if (this.hasNext) {
                    consumer.accept(e);
                    this.hasNext = false;
                }
            }
        };
    }
    
    static <T> Spliterator<T> singletonSpliterator(final T t) {
        return new Spliterator<T>() {
            long est = 1L;
            
            @Override
            public Spliterator<T> trySplit() {
                return null;
            }
            
            @Override
            public boolean tryAdvance(Consumer<? super T> consumer) {
                Objects.requireNonNull(consumer);
                if (this.est > 0L) {
                    --this.est;
                    consumer.accept(t);
                    return true;
                }
                return false;
            }
            
            @Override
            public void forEachRemaining(Consumer<? super T> consumer) {
                this.tryAdvance(consumer);
            }
            
            @Override
            public long estimateSize() {
                return this.est;
            }
            
            @Override
            public int characteristics() {
                return ((t != null) ? 256 : 0) | 0x40 | 0x4000 | 0x400 | 0x1 | 0x10;
            }
        };
    }
    
    public static <T> List<T> singletonList(T t) {
        return new Collections.SingletonList(t);
    }
    
    public static <K, V> Map<K, V> singletonMap(K k, V v) {
        return new Collections.SingletonMap(k, v);
    }
    
    public static <T> List<T> nCopies(int i, T t) {
        if (i < 0) {
            throw new IllegalArgumentException("List length = " + i);
        }
        return new Collections.CopiesList(i, t);
    }
    
    public static <T> Comparator<T> reverseOrder() {
        return Collections.ReverseComparator.REVERSE_ORDER;
    }
    
    public static <T> Comparator<T> reverseOrder(Comparator<T> comparator) {
        if (comparator == null) {
            return reverseOrder();
        }
        if ((comparator instanceof Collections.ReverseComparator2)) {
            return ((Collections.ReverseComparator2)comparator).cmp;
        }
        return new Collections.ReverseComparator2(comparator);
    }
    
    public static <T> Enumeration<T> enumeration(final Collection<T> collection) {
        return new Enumeration<T>() {
            private final Iterator<T> i = collection.iterator();
            
            @Override
            public boolean hasMoreElements() {
                return this.i.hasNext();
            }
            
            @Override
            public T nextElement() {
                return this.i.next();
            }
        };
    }
    
    public static <T> ArrayList<T> list(Enumeration<T> enumeration) {
        ArrayList<Object> list = new ArrayList();
        while (enumeration.hasMoreElements()) {
            list.add(enumeration.nextElement());
        }
        return list;
    }
    
    static boolean eq(Object o, Object obj) {
        return (o == null) ? (obj == null) : o.equals(obj);
    }
    
    public static int frequency(Collection<?> collection, Object o) {
        int n = 0;
        if (o == null) {
            Iterator<Object> iterator2 = collection.iterator();
            while (iterator2.hasNext()) {
                if (iterator2.next() == null) {
                    ++n;
                }
                else {
                    continue;
                }
            }
        }
        else {
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                if (o.equals(iterator.next())) {
                    ++n;
                }
                else {
                    continue;
                }
            }
        }
        return n;
    }
    
    public static boolean disjoint(Collection<?> collection, Collection<?> collection2) {
        if ((collection instanceof Set)) {
            collection = (collection2 = collection2);
        }
        else if (!(collection2 instanceof Set)) {
            int size = collection.size();
            int size2 = collection2.size();
            if (size == 0 || size2 == 0) {
                return true;
            }
            if (size > size2) {
                collection = (collection2 = collection2);
            }
        }
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            if (collection2.contains(iterator.next())) {
                return false;
            }
            continue;
        }
        return true;
    }
    
    @SafeVarargs
    public static <T> boolean addAll(Collection<? super T> collection, T... array) {
        boolean n = false;
        for (int length = array.length, i = 0; i < length; ++i) {
            n |= collection.add(array[i]);
        }
        return n;
    }
    
    public static <E> Set<E> newSetFromMap(Map<E, Boolean> map) {
        return new Collections.SetFromMap(map);
    }
    
    public static <T> Queue<T> asLifoQueue(Deque<T> deque) {
        return new Collections.AsLIFOQueue(deque);
    }
    
    static {
        EMPTY_SET = new Collections.EmptySet();
        EMPTY_LIST = new Collections.EmptyList();
        EMPTY_MAP = new Collections.EmptyMap();
    }
    
    static class UnmodifiableCollection<E> implements Collection<E>, Serializable
    {
        private static final long serialVersionUID = 1820017752578914078L;
        final Collection<? extends E> c;
        
        UnmodifiableCollection(Collection<? extends E> c) {
            if (c == null) {
                throw new NullPointerException();
            }
            this.c = c;
            return;
        }
        
        @Override
        public int size() {
            return this.c.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.c.isEmpty();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.c.contains(o);
        }
        
        @Override
        public Object[] toArray() {
            return this.c.toArray();
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            return this.c.toArray(array);
        }
        
        @Override
        public String toString() {
            return this.c.toString();
        }
        
        @Override
        public Iterator<E> iterator() {
            return new Iterator<E>() {
                private final Iterator<? extends E> i = Collections.UnmodifiableCollection.this.c.iterator();
                
                @Override
                public boolean hasNext() {
                    return this.i.hasNext();
                }
                
                @Override
                public E next() {
                    return this.i.next();
                }
                
                @Override
                public void remove() {
                    throw new UnsupportedOperationException();
                }
                
                @Override
                public void forEachRemaining(Consumer<? super E> consumer) {
                    this.i.forEachRemaining(consumer);
                }
            };
        }
        
        @Override
        public boolean add(E e) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean remove(Object o) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return this.c.containsAll(collection);
        }
        
        @Override
        public boolean addAll(Collection<? extends E> collection) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean retainAll(Collection<?> collection) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void clear() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void forEach(Consumer<? super E> action) {
            this.c.forEach(action);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.c.spliterator();
        }
        
        @Override
        public Stream<E> stream() {
            return this.c.stream();
        }
        
        @Override
        public Stream<E> parallelStream() {
            return this.c.parallelStream();
        }
    }
    
    static class UnmodifiableSet<E> extends Collections.UnmodifiableCollection<E> implements Set<E>, Serializable
    {
        private static final long serialVersionUID = -9215047833775013803L;
        
        UnmodifiableSet(Set<? extends E> set) {
            super(set);
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.c.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.c.hashCode();
        }
    }
    
    static class UnmodifiableSortedSet<E> extends Collections.UnmodifiableSet<E> implements SortedSet<E>, Serializable
    {
        private static final long serialVersionUID = -4929149591599911165L;
        private final SortedSet<E> ss;
        
        UnmodifiableSortedSet(SortedSet<E> ss) {
            super(ss);
            this.ss = ss;
        }
        
        @Override
        public Comparator<? super E> comparator() {
            return this.ss.comparator();
        }
        
        @Override
        public SortedSet<E> subSet(E e, E e2) {
            return new Collections.UnmodifiableSortedSet(this.ss.subSet(e, e2));
        }
        
        @Override
        public SortedSet<E> headSet(E e) {
            return new Collections.UnmodifiableSortedSet(this.ss.headSet(e));
        }
        
        @Override
        public SortedSet<E> tailSet(E e) {
            return new Collections.UnmodifiableSortedSet(this.ss.tailSet(e));
        }
        
        @Override
        public E first() {
            return this.ss.first();
        }
        
        @Override
        public E last() {
            return this.ss.last();
        }
    }
    
    static class UnmodifiableNavigableSet<E> extends Collections.UnmodifiableSortedSet<E> implements NavigableSet<E>, Serializable
    {
        private static final long serialVersionUID = -6027448201786391929L;
        private static final NavigableSet<?> EMPTY_NAVIGABLE_SET;
        private final NavigableSet<E> ns;
        
        UnmodifiableNavigableSet(NavigableSet<E> ns) {
            super(ns);
            this.ns = ns;
        }
        
        @Override
        public E lower(E e) {
            return this.ns.lower(e);
        }
        
        @Override
        public E floor(E e) {
            return this.ns.floor(e);
        }
        
        @Override
        public E ceiling(E e) {
            return this.ns.ceiling(e);
        }
        
        @Override
        public E higher(E e) {
            return this.ns.higher(e);
        }
        
        @Override
        public E pollFirst() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public E pollLast() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public NavigableSet<E> descendingSet() {
            return new Collections.UnmodifiableNavigableSet(this.ns.descendingSet());
        }
        
        @Override
        public Iterator<E> descendingIterator() {
            return this.descendingSet().iterator();
        }
        
        @Override
        public NavigableSet<E> subSet(E e, boolean b, E e2, boolean b2) {
            return new Collections.UnmodifiableNavigableSet(this.ns.subSet(e, b, e2, b2));
        }
        
        @Override
        public NavigableSet<E> headSet(E e, boolean b) {
            return new Collections.UnmodifiableNavigableSet(this.ns.headSet(e, b));
        }
        
        @Override
        public NavigableSet<E> tailSet(E e, boolean b) {
            return new Collections.UnmodifiableNavigableSet(this.ns.tailSet(e, b));
        }
        
        static {
            EMPTY_NAVIGABLE_SET = new Collections.UnmodifiableNavigableSet.EmptyNavigableSet();
        }
        
        private static class EmptyNavigableSet<E> extends Collections.UnmodifiableNavigableSet<E> implements Serializable
        {
            private static final long serialVersionUID = -6291252904449939134L;
            
            public EmptyNavigableSet() {
                super(new TreeSet());
            }
            
            private Object readResolve() {
                return Collections.UnmodifiableNavigableSet.EMPTY_NAVIGABLE_SET;
            }
        }
    }
    
    static class UnmodifiableList<E> extends Collections.UnmodifiableCollection<E> implements List<E>
    {
        private static final long serialVersionUID = -283967356065247728L;
        final List<? extends E> list;
        
        UnmodifiableList(List<? extends E> list) {
            super(list);
            this.list = list;
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.list.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.list.hashCode();
        }
        
        @Override
        public E get(int n) {
            return this.list.get(n);
        }
        
        @Override
        public E set(int n, E e) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void add(int n, E e) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public E remove(int n) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public int indexOf(Object o) {
            return this.list.indexOf(o);
        }
        
        @Override
        public int lastIndexOf(Object o) {
            return this.list.lastIndexOf(o);
        }
        
        @Override
        public boolean addAll(int n, Collection<? extends E> collection) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void replaceAll(UnaryOperator<E> unaryOperator) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void sort(Comparator<? super E> comparator) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public ListIterator<E> listIterator() {
            return this.listIterator(0);
        }
        
        @Override
        public ListIterator<E> listIterator(int n) {
            return new ListIterator<E>() {
                private final ListIterator<? extends E> i = Collections.UnmodifiableList.this.list.listIterator(n);
                
                @Override
                public boolean hasNext() {
                    return this.i.hasNext();
                }
                
                @Override
                public E next() {
                    return this.i.next();
                }
                
                @Override
                public boolean hasPrevious() {
                    return this.i.hasPrevious();
                }
                
                @Override
                public E previous() {
                    return this.i.previous();
                }
                
                @Override
                public int nextIndex() {
                    return this.i.nextIndex();
                }
                
                @Override
                public int previousIndex() {
                    return this.i.previousIndex();
                }
                
                @Override
                public void remove() {
                    throw new UnsupportedOperationException();
                }
                
                @Override
                public void set(E e) {
                    throw new UnsupportedOperationException();
                }
                
                @Override
                public void add(E e) {
                    throw new UnsupportedOperationException();
                }
                
                @Override
                public void forEachRemaining(Consumer<? super E> consumer) {
                    this.i.forEachRemaining(consumer);
                }
            };
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            return new Collections.UnmodifiableList(this.list.subList(n, n2));
        }
        
        private Object readResolve() {
            return (this.list instanceof RandomAccess) ? new Collections.UnmodifiableRandomAccessList(this.list) : this;
        }
    }
    
    static class UnmodifiableRandomAccessList<E> extends Collections.UnmodifiableList<E> implements RandomAccess
    {
        private static final long serialVersionUID = -2542308836966382001L;
        
        UnmodifiableRandomAccessList(List<? extends E> list) {
            super(list);
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            return new Collections.UnmodifiableRandomAccessList(this.list.subList(n, n2));
        }
        
        private Object writeReplace() {
            return new Collections.UnmodifiableList(this.list);
        }
    }
    
    private static class UnmodifiableMap<K, V> implements Map<K, V>, Serializable
    {
        private static final long serialVersionUID = -1034234728574286014L;
        private final Map<? extends K, ? extends V> m;
        private transient Set<K> keySet;
        private transient Set<Map.Entry<K, V>> entrySet;
        private transient Collection<V> values;
        
        UnmodifiableMap(Map<? extends K, ? extends V> m) {
            if (m == null) {
                throw new NullPointerException();
            }
            this.m = m;
            return;
        }
        
        @Override
        public int size() {
            return this.m.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.m.isEmpty();
        }
        
        @Override
        public boolean containsKey(Object o) {
            return this.m.containsKey(o);
        }
        
        @Override
        public boolean containsValue(Object o) {
            return this.m.containsValue(o);
        }
        
        @Override
        public V get(Object o) {
            return this.m.get(o);
        }
        
        @Override
        public V put(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V remove(Object o) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void putAll(Map<? extends K, ? extends V> map) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void clear() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public Set<K> keySet() {
            if (this.keySet == null) {
                this.keySet = Collections.unmodifiableSet(this.m.keySet());
            }
            return this.keySet;
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            if (this.entrySet == null) {
                this.entrySet = new Collections.UnmodifiableMap.UnmodifiableEntrySet(this.m.entrySet());
            }
            return this.entrySet;
        }
        
        @Override
        public Collection<V> values() {
            if (this.values == null) {
                this.values = Collections.unmodifiableCollection(this.m.values());
            }
            return this.values;
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.m.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.m.hashCode();
        }
        
        @Override
        public String toString() {
            return this.m.toString();
        }
        
        @Override
        public V getOrDefault(Object o, V v) {
            return this.m.getOrDefault(o, v);
        }
        
        @Override
        public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
            this.m.forEach(biConsumer);
        }
        
        @Override
        public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V putIfAbsent(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean remove(Object o, Object o2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean replace(K k, V v, V v2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V replace(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        static class UnmodifiableEntrySet<K, V> extends Collections.UnmodifiableSet<Map.Entry<K, V>>
        {
            private static final long serialVersionUID = 7854390611657943733L;
            
            UnmodifiableEntrySet(Set<? extends Map.Entry<? extends K, ? extends V>> set) {
                super(set);
            }
            
            static <K, V> Consumer<Map.Entry<K, V>> entryConsumer(Consumer<? super Map.Entry<K, V>> consumer) {
                return entry/* java.util.Map.Entry, */ -> p0.accept(new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntry(entry));
            }
            
            @Override
            public void forEach(Consumer<? super Map.Entry<K, V>> consumer) {
                Objects.requireNonNull(consumer);
                this.c.forEach(entryConsumer(consumer));
            }
            
            @Override
            public Spliterator<Map.Entry<K, V>> spliterator() {
                return new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntrySetSpliterator(this.c.spliterator());
            }
            
            @Override
            public Stream<Map.Entry<K, V>> stream() {
                return StreamSupport.stream(this.spliterator(), false);
            }
            
            @Override
            public Stream<Map.Entry<K, V>> parallelStream() {
                return StreamSupport.stream(this.spliterator(), true);
            }
            
            @Override
            public Iterator<Map.Entry<K, V>> iterator() {
                return new Iterator<Map.Entry<K, V>>() {
                    private final Iterator<? extends Map.Entry<? extends K, ? extends V>> i = Collections.UnmodifiableMap.UnmodifiableEntrySet.this.c.iterator();
                    
                    @Override
                    public boolean hasNext() {
                        return this.i.hasNext();
                    }
                    
                    @Override
                    public Map.Entry<K, V> next() {
                        return new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntry((Map.Entry)this.i.next());
                    }
                    
                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
            
            @Override
            public Object[] toArray() {
                Object[] array = this.c.toArray();
                for (int i = 0; i < array.length; ++i) {
                    array[i] = new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntry((Map.Entry)array[i]);
                }
                return array;
            }
            
            @Override
            public <T> T[] toArray(T[] array) {
                Object[] array2 = this.c.toArray((array.length == 0) ? array : Arrays.copyOf(array, 0));
                for (int i = 0; i < array2.length; ++i) {
                    array2[i] = new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntry((Map.Entry)array2[i]);
                }
                if (array2.length > array.length) {
                    return (T[])array2;
                }
                System.arraycopy(array2, 0, array, 0, array2.length);
                if (array.length > array2.length) {
                    array[array2.length] = null;
                    return array;
                }
                return array;
            }
            
            @Override
            public boolean contains(Object o) {
                return o instanceof Map.Entry && this.c.contains(new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntry((Map.Entry)o));
            }
            
            @Override
            public boolean containsAll(Collection<?> collection) {
                Iterator<Object> iterator = collection.iterator();
                while (iterator.hasNext()) {
                    if (!this.contains(iterator.next())) {
                        return false;
                    }
                    continue;
                }
                return true;
            }
            
            @Override
            public boolean equals(Object o) {
                if (o == this) {
                    return true;
                }
                if (!(o instanceof Set)) {
                    return false;
                }
                Set<Object> set = (Set)o;
                return set.size() == this.c.size() && this.containsAll(set);
            }
            
            static final class UnmodifiableEntrySetSpliterator<K, V> implements Spliterator<Map.Entry<K, V>>
            {
                final Spliterator<Map.Entry<K, V>> s;
                
                UnmodifiableEntrySetSpliterator(Spliterator<Map.Entry<K, V>> s) {
                    this.s = s;
                }
                
                @Override
                public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
                    Objects.requireNonNull(consumer);
                    return this.s.tryAdvance(Collections.UnmodifiableMap.UnmodifiableEntrySet.entryConsumer(consumer));
                }
                
                @Override
                public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
                    Objects.requireNonNull(consumer);
                    this.s.forEachRemaining(Collections.UnmodifiableMap.UnmodifiableEntrySet.entryConsumer(consumer));
                }
                
                @Override
                public Spliterator<Map.Entry<K, V>> trySplit() {
                    Spliterator<Map.Entry<K, V>> trySplit = this.s.trySplit();
                    return (trySplit == null) ? null : new Collections.UnmodifiableMap.UnmodifiableEntrySet.UnmodifiableEntrySetSpliterator(trySplit);
                }
                
                @Override
                public long estimateSize() {
                    return this.s.estimateSize();
                }
                
                @Override
                public long getExactSizeIfKnown() {
                    return this.s.getExactSizeIfKnown();
                }
                
                @Override
                public int characteristics() {
                    return this.s.characteristics();
                }
                
                @Override
                public boolean hasCharacteristics(int n) {
                    return this.s.hasCharacteristics(n);
                }
                
                @Override
                public Comparator<? super Map.Entry<K, V>> getComparator() {
                    return this.s.getComparator();
                }
            }
            
            private static class UnmodifiableEntry<K, V> implements Map.Entry<K, V>
            {
                private Map.Entry<? extends K, ? extends V> e;
                
                UnmodifiableEntry(Map.Entry<? extends K, ? extends V> entry) {
                    this.e = (Map.Entry<? extends K, ? extends V>)Objects.requireNonNull(entry);
                }
                
                @Override
                public K getKey() {
                    return this.e.getKey();
                }
                
                @Override
                public V getValue() {
                    return this.e.getValue();
                }
                
                @Override
                public V setValue(V v) {
                    throw new UnsupportedOperationException();
                }
                
                @Override
                public int hashCode() {
                    return this.e.hashCode();
                }
                
                @Override
                public boolean equals(Object o) {
                    if (this == o) {
                        return true;
                    }
                    if (!(o instanceof Map.Entry)) {
                        return false;
                    }
                    Map.Entry<Object, Object> entry = (Map.Entry)o;
                    return Collections.eq(this.e.getKey(), entry.getKey()) && Collections.eq(this.e.getValue(), entry.getValue());
                }
                
                @Override
                public String toString() {
                    return this.e.toString();
                }
            }
        }
    }
    
    static class UnmodifiableSortedMap<K, V> extends Collections.UnmodifiableMap<K, V> implements SortedMap<K, V>, Serializable
    {
        private static final long serialVersionUID = -8806743815996713206L;
        private final SortedMap<K, ? extends V> sm;
        
        UnmodifiableSortedMap(SortedMap<K, ? extends V> sm) {
            super(sm);
            this.sm = sm;
        }
        
        @Override
        public Comparator<? super K> comparator() {
            return this.sm.comparator();
        }
        
        @Override
        public SortedMap<K, V> subMap(K k, K i) {
            return new Collections.UnmodifiableSortedMap(this.sm.subMap(k, i));
        }
        
        @Override
        public SortedMap<K, V> headMap(K k) {
            return new Collections.UnmodifiableSortedMap(this.sm.headMap(k));
        }
        
        @Override
        public SortedMap<K, V> tailMap(K k) {
            return new Collections.UnmodifiableSortedMap(this.sm.tailMap(k));
        }
        
        @Override
        public K firstKey() {
            return this.sm.firstKey();
        }
        
        @Override
        public K lastKey() {
            return this.sm.lastKey();
        }
    }
    
    static class UnmodifiableNavigableMap<K, V> extends Collections.UnmodifiableSortedMap<K, V> implements NavigableMap<K, V>, Serializable
    {
        private static final long serialVersionUID = -4858195264774772197L;
        private static final Collections.UnmodifiableNavigableMap.EmptyNavigableMap<?, ?> EMPTY_NAVIGABLE_MAP;
        private final NavigableMap<K, ? extends V> nm;
        
        UnmodifiableNavigableMap(NavigableMap<K, ? extends V> nm) {
            super(nm);
            this.nm = nm;
        }
        
        @Override
        public K lowerKey(K k) {
            return this.nm.lowerKey(k);
        }
        
        @Override
        public K floorKey(K k) {
            return this.nm.floorKey(k);
        }
        
        @Override
        public K ceilingKey(K k) {
            return this.nm.ceilingKey(k);
        }
        
        @Override
        public K higherKey(K k) {
            return this.nm.higherKey(k);
        }
        
        @Override
        public Map.Entry<K, V> lowerEntry(K k) {
            Map.Entry<K, Object> lowerEntry = this.nm.lowerEntry(k);
            return (null != lowerEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(lowerEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> floorEntry(K k) {
            Map.Entry<K, Object> floorEntry = this.nm.floorEntry(k);
            return (null != floorEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(floorEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> ceilingEntry(K k) {
            Map.Entry<K, Object> ceilingEntry = this.nm.ceilingEntry(k);
            return (null != ceilingEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(ceilingEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> higherEntry(K k) {
            Map.Entry<K, Object> higherEntry = this.nm.higherEntry(k);
            return (null != higherEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(higherEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> firstEntry() {
            Map.Entry<K, Object> firstEntry = this.nm.firstEntry();
            return (null != firstEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(firstEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> lastEntry() {
            Map.Entry<K, Object> lastEntry = this.nm.lastEntry();
            return (null != lastEntry) ? new UnmodifiableEntrySet.UnmodifiableEntry(lastEntry) : null;
        }
        
        @Override
        public Map.Entry<K, V> pollFirstEntry() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public Map.Entry<K, V> pollLastEntry() {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public NavigableMap<K, V> descendingMap() {
            return Collections.unmodifiableNavigableMap(this.nm.descendingMap());
        }
        
        @Override
        public NavigableSet<K> navigableKeySet() {
            return Collections.unmodifiableNavigableSet(this.nm.navigableKeySet());
        }
        
        @Override
        public NavigableSet<K> descendingKeySet() {
            return Collections.unmodifiableNavigableSet(this.nm.descendingKeySet());
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
            return Collections.unmodifiableNavigableMap(this.nm.subMap(k, b, i, b2));
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k, boolean b) {
            return Collections.unmodifiableNavigableMap(this.nm.headMap(k, b));
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k, boolean b) {
            return Collections.unmodifiableNavigableMap(this.nm.tailMap(k, b));
        }
        
        static {
            EMPTY_NAVIGABLE_MAP = new Collections.UnmodifiableNavigableMap.EmptyNavigableMap();
        }
        
        private static class EmptyNavigableMap<K, V> extends Collections.UnmodifiableNavigableMap<K, V> implements Serializable
        {
            private static final long serialVersionUID = -2239321462712562324L;
            
            EmptyNavigableMap() {
                super(new TreeMap());
            }
            
            @Override
            public NavigableSet<K> navigableKeySet() {
                return Collections.emptyNavigableSet();
            }
            
            private Object readResolve() {
                return Collections.UnmodifiableNavigableMap.EMPTY_NAVIGABLE_MAP;
            }
        }
    }
    
    static class SynchronizedCollection<E> implements Collection<E>, Serializable
    {
        private static final long serialVersionUID = 3053995032091335093L;
        final Collection<E> c;
        final Object mutex;
        
        SynchronizedCollection(Collection<E> collection) {
            this.c = (Collection<E>)Objects.requireNonNull(collection);
            this.mutex = this;
        }
        
        SynchronizedCollection(Collection<E> collection, Object o) {
            this.c = (Collection<E>)Objects.requireNonNull(collection);
            this.mutex = Objects.requireNonNull(o);
        }
        
        @Override
        public int size() {
            synchronized (this.mutex) {
                return this.c.size();
            }
        }
        
        @Override
        public boolean isEmpty() {
            synchronized (this.mutex) {
                return this.c.isEmpty();
            }
        }
        
        @Override
        public boolean contains(Object o) {
            synchronized (this.mutex) {
                return this.c.contains(o);
            }
        }
        
        @Override
        public Object[] toArray() {
            synchronized (this.mutex) {
                return this.c.toArray();
            }
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            synchronized (this.mutex) {
                return this.c.toArray(array);
            }
        }
        
        @Override
        public Iterator<E> iterator() {
            return this.c.iterator();
        }
        
        @Override
        public boolean add(E e) {
            synchronized (this.mutex) {
                return this.c.add(e);
            }
        }
        
        @Override
        public boolean remove(Object o) {
            synchronized (this.mutex) {
                return this.c.remove(o);
            }
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            synchronized (this.mutex) {
                return this.c.containsAll(collection);
            }
        }
        
        @Override
        public boolean addAll(Collection<? extends E> collection) {
            synchronized (this.mutex) {
                return this.c.addAll(collection);
            }
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            synchronized (this.mutex) {
                return this.c.removeAll(collection);
            }
        }
        
        @Override
        public boolean retainAll(Collection<?> collection) {
            synchronized (this.mutex) {
                return this.c.retainAll(collection);
            }
        }
        
        @Override
        public void clear() {
            synchronized (this.mutex) {
                this.c.clear();
            }
        }
        
        @Override
        public String toString() {
            synchronized (this.mutex) {
                return this.c.toString();
            }
        }
        
        @Override
        public void forEach(Consumer<? super E> action) {
            synchronized (this.mutex) {
                this.c.forEach(action);
            }
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            synchronized (this.mutex) {
                return this.c.removeIf(predicate);
            }
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.c.spliterator();
        }
        
        @Override
        public Stream<E> stream() {
            return this.c.stream();
        }
        
        @Override
        public Stream<E> parallelStream() {
            return this.c.parallelStream();
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            synchronized (this.mutex) {
                objectOutputStream.defaultWriteObject();
            }
        }
    }
    
    static class SynchronizedSet<E> extends Collections.SynchronizedCollection<E> implements Set<E>
    {
        private static final long serialVersionUID = 487447009682186044L;
        
        SynchronizedSet(Set<E> set) {
            super(set);
        }
        
        SynchronizedSet(Set<E> set, Object o) {
            super(set, o);
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            synchronized (this.mutex) {
                return this.c.equals(o);
            }
        }
        
        @Override
        public int hashCode() {
            synchronized (this.mutex) {
                return this.c.hashCode();
            }
        }
    }
    
    static class SynchronizedSortedSet<E> extends Collections.SynchronizedSet<E> implements SortedSet<E>
    {
        private static final long serialVersionUID = 8695801310862127406L;
        private final SortedSet<E> ss;
        
        SynchronizedSortedSet(SortedSet<E> ss) {
            super(ss);
            this.ss = ss;
        }
        
        SynchronizedSortedSet(SortedSet<E> ss, Object o) {
            super(ss, o);
            this.ss = ss;
        }
        
        @Override
        public Comparator<? super E> comparator() {
            synchronized (this.mutex) {
                return this.ss.comparator();
            }
        }
        
        @Override
        public SortedSet<E> subSet(E e, E e2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedSet(this.ss.subSet(e, e2), this.mutex);
            }
        }
        
        @Override
        public SortedSet<E> headSet(E e) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedSet(this.ss.headSet(e), this.mutex);
            }
        }
        
        @Override
        public SortedSet<E> tailSet(E e) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedSet(this.ss.tailSet(e), this.mutex);
            }
        }
        
        @Override
        public E first() {
            synchronized (this.mutex) {
                return this.ss.first();
            }
        }
        
        @Override
        public E last() {
            synchronized (this.mutex) {
                return this.ss.last();
            }
        }
    }
    
    static class SynchronizedNavigableSet<E> extends Collections.SynchronizedSortedSet<E> implements NavigableSet<E>
    {
        private static final long serialVersionUID = -5505529816273629798L;
        private final NavigableSet<E> ns;
        
        SynchronizedNavigableSet(NavigableSet<E> ns) {
            super(ns);
            this.ns = ns;
        }
        
        SynchronizedNavigableSet(NavigableSet<E> ns, Object o) {
            super(ns, o);
            this.ns = ns;
        }
        
        @Override
        public E lower(E e) {
            synchronized (this.mutex) {
                return this.ns.lower(e);
            }
        }
        
        @Override
        public E floor(E e) {
            synchronized (this.mutex) {
                return this.ns.floor(e);
            }
        }
        
        @Override
        public E ceiling(E e) {
            synchronized (this.mutex) {
                return this.ns.ceiling(e);
            }
        }
        
        @Override
        public E higher(E e) {
            synchronized (this.mutex) {
                return this.ns.higher(e);
            }
        }
        
        @Override
        public E pollFirst() {
            synchronized (this.mutex) {
                return this.ns.pollFirst();
            }
        }
        
        @Override
        public E pollLast() {
            synchronized (this.mutex) {
                return this.ns.pollLast();
            }
        }
        
        @Override
        public NavigableSet<E> descendingSet() {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.descendingSet(), this.mutex);
            }
        }
        
        @Override
        public Iterator<E> descendingIterator() {
            synchronized (this.mutex) {
                return this.descendingSet().iterator();
            }
        }
        
        @Override
        public NavigableSet<E> subSet(E e, E e2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.subSet(e, true, e2, false), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<E> headSet(E e) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.headSet(e, false), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<E> tailSet(E e) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.tailSet(e, true), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<E> subSet(E e, boolean b, E e2, boolean b2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.subSet(e, b, e2, b2), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<E> headSet(E e, boolean b) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.headSet(e, b), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<E> tailSet(E e, boolean b) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.ns.tailSet(e, b), this.mutex);
            }
        }
    }
    
    static class SynchronizedList<E> extends Collections.SynchronizedCollection<E> implements List<E>
    {
        private static final long serialVersionUID = -7754090372962971524L;
        final List<E> list;
        
        SynchronizedList(List<E> list) {
            super(list);
            this.list = list;
        }
        
        SynchronizedList(List<E> list, Object o) {
            super(list, o);
            this.list = list;
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            synchronized (this.mutex) {
                return this.list.equals(o);
            }
        }
        
        @Override
        public int hashCode() {
            synchronized (this.mutex) {
                return this.list.hashCode();
            }
        }
        
        @Override
        public E get(int n) {
            synchronized (this.mutex) {
                return this.list.get(n);
            }
        }
        
        @Override
        public E set(int n, E e) {
            synchronized (this.mutex) {
                return this.list.set(n, e);
            }
        }
        
        @Override
        public void add(int n, E e) {
            synchronized (this.mutex) {
                this.list.add(n, e);
            }
        }
        
        @Override
        public E remove(int n) {
            synchronized (this.mutex) {
                return this.list.remove(n);
            }
        }
        
        @Override
        public int indexOf(Object o) {
            synchronized (this.mutex) {
                return this.list.indexOf(o);
            }
        }
        
        @Override
        public int lastIndexOf(Object o) {
            synchronized (this.mutex) {
                return this.list.lastIndexOf(o);
            }
        }
        
        @Override
        public boolean addAll(int n, Collection<? extends E> collection) {
            synchronized (this.mutex) {
                return this.list.addAll(n, collection);
            }
        }
        
        @Override
        public ListIterator<E> listIterator() {
            return this.list.listIterator();
        }
        
        @Override
        public ListIterator<E> listIterator(int n) {
            return this.list.listIterator(n);
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedList(this.list.subList(n, n2), this.mutex);
            }
        }
        
        @Override
        public void replaceAll(UnaryOperator<E> unaryOperator) {
            synchronized (this.mutex) {
                this.list.replaceAll(unaryOperator);
            }
        }
        
        @Override
        public void sort(Comparator<? super E> comparator) {
            synchronized (this.mutex) {
                this.list.sort(comparator);
            }
        }
        
        private Object readResolve() {
            return (this.list instanceof RandomAccess) ? new Collections.SynchronizedRandomAccessList(this.list) : this;
        }
    }
    
    static class SynchronizedRandomAccessList<E> extends Collections.SynchronizedList<E> implements RandomAccess
    {
        private static final long serialVersionUID = 1530674583602358482L;
        
        SynchronizedRandomAccessList(List<E> list) {
            super(list);
        }
        
        SynchronizedRandomAccessList(List<E> list, Object o) {
            super(list, o);
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedRandomAccessList(this.list.subList(n, n2), this.mutex);
            }
        }
        
        private Object writeReplace() {
            return new Collections.SynchronizedList(this.list);
        }
    }
    
    private static class SynchronizedMap<K, V> implements Map<K, V>, Serializable
    {
        private static final long serialVersionUID = 1978198479659022715L;
        private final Map<K, V> m;
        final Object mutex;
        private transient Set<K> keySet;
        private transient Set<Map.Entry<K, V>> entrySet;
        private transient Collection<V> values;
        
        SynchronizedMap(Map<K, V> map) {
            this.m = (Map<K, V>)Objects.requireNonNull(map);
            this.mutex = this;
        }
        
        SynchronizedMap(Map<K, V> m, Object mutex) {
            this.m = m;
            this.mutex = mutex;
        }
        
        @Override
        public int size() {
            synchronized (this.mutex) {
                return this.m.size();
            }
        }
        
        @Override
        public boolean isEmpty() {
            synchronized (this.mutex) {
                return this.m.isEmpty();
            }
        }
        
        @Override
        public boolean containsKey(Object o) {
            synchronized (this.mutex) {
                return this.m.containsKey(o);
            }
        }
        
        @Override
        public boolean containsValue(Object o) {
            synchronized (this.mutex) {
                return this.m.containsValue(o);
            }
        }
        
        @Override
        public V get(Object o) {
            synchronized (this.mutex) {
                return this.m.get(o);
            }
        }
        
        @Override
        public V put(K k, V v) {
            synchronized (this.mutex) {
                return this.m.put(k, v);
            }
        }
        
        @Override
        public V remove(Object o) {
            synchronized (this.mutex) {
                return this.m.remove(o);
            }
        }
        
        @Override
        public void putAll(Map<? extends K, ? extends V> map) {
            synchronized (this.mutex) {
                this.m.putAll(map);
            }
        }
        
        @Override
        public void clear() {
            synchronized (this.mutex) {
                this.m.clear();
            }
        }
        
        @Override
        public Set<K> keySet() {
            synchronized (this.mutex) {
                if (this.keySet == null) {
                    this.keySet = new Collections.SynchronizedSet(this.m.keySet(), this.mutex);
                }
                return this.keySet;
            }
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            synchronized (this.mutex) {
                if (this.entrySet == null) {
                    this.entrySet = new Collections.SynchronizedSet(this.m.entrySet(), this.mutex);
                }
                return this.entrySet;
            }
        }
        
        @Override
        public Collection<V> values() {
            synchronized (this.mutex) {
                if (this.values == null) {
                    this.values = new Collections.SynchronizedCollection(this.m.values(), this.mutex);
                }
                return this.values;
            }
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            synchronized (this.mutex) {
                return this.m.equals(o);
            }
        }
        
        @Override
        public int hashCode() {
            synchronized (this.mutex) {
                return this.m.hashCode();
            }
        }
        
        @Override
        public String toString() {
            synchronized (this.mutex) {
                return this.m.toString();
            }
        }
        
        @Override
        public V getOrDefault(Object o, V v) {
            synchronized (this.mutex) {
                return this.m.getOrDefault(o, v);
            }
        }
        
        @Override
        public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
            synchronized (this.mutex) {
                this.m.forEach(biConsumer);
            }
        }
        
        @Override
        public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            synchronized (this.mutex) {
                this.m.replaceAll(biFunction);
            }
        }
        
        @Override
        public V putIfAbsent(K k, V v) {
            synchronized (this.mutex) {
                return this.m.putIfAbsent(k, v);
            }
        }
        
        @Override
        public boolean remove(Object o, Object o2) {
            synchronized (this.mutex) {
                return this.m.remove(o, o2);
            }
        }
        
        @Override
        public boolean replace(K k, V v, V v2) {
            synchronized (this.mutex) {
                return this.m.replace(k, v, v2);
            }
        }
        
        @Override
        public V replace(K k, V v) {
            synchronized (this.mutex) {
                return this.m.replace(k, v);
            }
        }
        
        @Override
        public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
            synchronized (this.mutex) {
                return this.m.computeIfAbsent(k, function);
            }
        }
        
        @Override
        public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            synchronized (this.mutex) {
                return this.m.computeIfPresent(k, biFunction);
            }
        }
        
        @Override
        public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            synchronized (this.mutex) {
                return this.m.compute(k, biFunction);
            }
        }
        
        @Override
        public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
            synchronized (this.mutex) {
                return this.m.merge(k, v, biFunction);
            }
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            synchronized (this.mutex) {
                objectOutputStream.defaultWriteObject();
            }
        }
    }
    
    static class SynchronizedSortedMap<K, V> extends Collections.SynchronizedMap<K, V> implements SortedMap<K, V>
    {
        private static final long serialVersionUID = -8798146769416483793L;
        private final SortedMap<K, V> sm;
        
        SynchronizedSortedMap(SortedMap<K, V> sm) {
            super(sm);
            this.sm = sm;
        }
        
        SynchronizedSortedMap(SortedMap<K, V> sm, Object o) {
            super(sm, o);
            this.sm = sm;
        }
        
        @Override
        public Comparator<? super K> comparator() {
            synchronized (this.mutex) {
                return this.sm.comparator();
            }
        }
        
        @Override
        public SortedMap<K, V> subMap(K k, K i) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedMap(this.sm.subMap(k, i), this.mutex);
            }
        }
        
        @Override
        public SortedMap<K, V> headMap(K k) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedMap(this.sm.headMap(k), this.mutex);
            }
        }
        
        @Override
        public SortedMap<K, V> tailMap(K k) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedSortedMap(this.sm.tailMap(k), this.mutex);
            }
        }
        
        @Override
        public K firstKey() {
            synchronized (this.mutex) {
                return this.sm.firstKey();
            }
        }
        
        @Override
        public K lastKey() {
            synchronized (this.mutex) {
                return this.sm.lastKey();
            }
        }
    }
    
    static class SynchronizedNavigableMap<K, V> extends Collections.SynchronizedSortedMap<K, V> implements NavigableMap<K, V>
    {
        private static final long serialVersionUID = 699392247599746807L;
        private final NavigableMap<K, V> nm;
        
        SynchronizedNavigableMap(NavigableMap<K, V> nm) {
            super(nm);
            this.nm = nm;
        }
        
        SynchronizedNavigableMap(NavigableMap<K, V> nm, Object o) {
            super(nm, o);
            this.nm = nm;
        }
        
        @Override
        public Map.Entry<K, V> lowerEntry(K k) {
            synchronized (this.mutex) {
                return this.nm.lowerEntry(k);
            }
        }
        
        @Override
        public K lowerKey(K k) {
            synchronized (this.mutex) {
                return this.nm.lowerKey(k);
            }
        }
        
        @Override
        public Map.Entry<K, V> floorEntry(K k) {
            synchronized (this.mutex) {
                return this.nm.floorEntry(k);
            }
        }
        
        @Override
        public K floorKey(K k) {
            synchronized (this.mutex) {
                return this.nm.floorKey(k);
            }
        }
        
        @Override
        public Map.Entry<K, V> ceilingEntry(K k) {
            synchronized (this.mutex) {
                return this.nm.ceilingEntry(k);
            }
        }
        
        @Override
        public K ceilingKey(K k) {
            synchronized (this.mutex) {
                return this.nm.ceilingKey(k);
            }
        }
        
        @Override
        public Map.Entry<K, V> higherEntry(K k) {
            synchronized (this.mutex) {
                return this.nm.higherEntry(k);
            }
        }
        
        @Override
        public K higherKey(K k) {
            synchronized (this.mutex) {
                return this.nm.higherKey(k);
            }
        }
        
        @Override
        public Map.Entry<K, V> firstEntry() {
            synchronized (this.mutex) {
                return this.nm.firstEntry();
            }
        }
        
        @Override
        public Map.Entry<K, V> lastEntry() {
            synchronized (this.mutex) {
                return this.nm.lastEntry();
            }
        }
        
        @Override
        public Map.Entry<K, V> pollFirstEntry() {
            synchronized (this.mutex) {
                return this.nm.pollFirstEntry();
            }
        }
        
        @Override
        public Map.Entry<K, V> pollLastEntry() {
            synchronized (this.mutex) {
                return this.nm.pollLastEntry();
            }
        }
        
        @Override
        public NavigableMap<K, V> descendingMap() {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.descendingMap(), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<K> keySet() {
            return this.navigableKeySet();
        }
        
        @Override
        public NavigableSet<K> navigableKeySet() {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.nm.navigableKeySet(), this.mutex);
            }
        }
        
        @Override
        public NavigableSet<K> descendingKeySet() {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableSet(this.nm.descendingKeySet(), this.mutex);
            }
        }
        
        @Override
        public SortedMap<K, V> subMap(K k, K i) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.subMap(k, true, i, false), this.mutex);
            }
        }
        
        @Override
        public SortedMap<K, V> headMap(K k) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.headMap(k, false), this.mutex);
            }
        }
        
        @Override
        public SortedMap<K, V> tailMap(K k) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.tailMap(k, true), this.mutex);
            }
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.subMap(k, b, i, b2), this.mutex);
            }
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k, boolean b) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.headMap(k, b), this.mutex);
            }
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k, boolean b) {
            synchronized (this.mutex) {
                return new Collections.SynchronizedNavigableMap(this.nm.tailMap(k, b), this.mutex);
            }
        }
    }
    
    static class CheckedCollection<E> implements Collection<E>, Serializable
    {
        private static final long serialVersionUID = 1578914078182001775L;
        final Collection<E> c;
        final Class<E> type;
        private E[] zeroLengthElementArray;
        
        E typeCheck(Object o) {
            if (o != null && !this.type.isInstance(o)) {
                throw new ClassCastException(this.badElementMsg(o));
            }
            return o;
        }
        
        private String badElementMsg(Object o) {
            return "Attempt to insert " + o.getClass() + " element into collection with element type " + this.type;
        }
        
        CheckedCollection(Collection<E> collection, Class<E> clazz) {
            this.c = (Collection<E>)Objects.requireNonNull(collection, "c");
            this.type = (Class<E>)Objects.requireNonNull(clazz, "type");
        }
        
        @Override
        public int size() {
            return this.c.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.c.isEmpty();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.c.contains(o);
        }
        
        @Override
        public Object[] toArray() {
            return this.c.toArray();
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            return this.c.toArray(array);
        }
        
        @Override
        public String toString() {
            return this.c.toString();
        }
        
        @Override
        public boolean remove(Object o) {
            return this.c.remove(o);
        }
        
        @Override
        public void clear() {
            this.c.clear();
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return this.c.containsAll(collection);
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            return this.c.removeAll(collection);
        }
        
        @Override
        public boolean retainAll(Collection<?> collection) {
            return this.c.retainAll(collection);
        }
        
        @Override
        public Iterator<E> iterator() {
            return new Iterator<E>() {
                
                @Override
                public boolean hasNext() {
                    return it.hasNext();
                }
                
                @Override
                public E next() {
                    return it.next();
                }
                
                @Override
                public void remove() {
                    it.remove();
                }
            };
        }
        
        @Override
        public boolean add(E e) {
            return this.c.add(this.typeCheck(e));
        }
        
        private E[] zeroLengthElementArray() {
            return (this.zeroLengthElementArray != null) ? this.zeroLengthElementArray : (this.zeroLengthElementArray = Collections.zeroLengthArray(this.type));
        }
        
        Collection<E> checkedCopyOf(Collection<? extends E> collection) {
            Object[] array;
            try {
                Object[] zeroLengthElementArray = this.zeroLengthElementArray();
                array = collection.toArray(zeroLengthElementArray);
                if (array.getClass() != zeroLengthElementArray.getClass()) {
                    array = Arrays.copyOf(array, array.length, zeroLengthElementArray.getClass());
                }
            }
            catch (ArrayStoreException ex) {
                array = (array = (Object[])collection.toArray().clone());
                for (int length = array.length, i = 0; i < length; ++i) {
                    this.typeCheck(array[i]);
                }
            }
            return Arrays.asList(array);
        }
        
        @Override
        public boolean addAll(Collection<? extends E> collection) {
            return this.c.addAll(this.checkedCopyOf(collection));
        }
        
        @Override
        public void forEach(Consumer<? super E> action) {
            this.c.forEach(action);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            return this.c.removeIf(predicate);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.c.spliterator();
        }
        
        @Override
        public Stream<E> stream() {
            return this.c.stream();
        }
        
        @Override
        public Stream<E> parallelStream() {
            return this.c.parallelStream();
        }
    }
    
    static class CheckedQueue<E> extends Collections.CheckedCollection<E> implements Queue<E>, Serializable
    {
        private static final long serialVersionUID = 1433151992604707767L;
        final Queue<E> queue;
        
        CheckedQueue(Queue<E> queue, Class<E> clazz) {
            super(queue, clazz);
            this.queue = queue;
        }
        
        @Override
        public E element() {
            return this.queue.element();
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.c.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.c.hashCode();
        }
        
        @Override
        public E peek() {
            return this.queue.peek();
        }
        
        @Override
        public E poll() {
            return this.queue.poll();
        }
        
        @Override
        public E remove() {
            return this.queue.remove();
        }
        
        @Override
        public boolean offer(E e) {
            return this.queue.offer(this.typeCheck(e));
        }
    }
    
    static class CheckedSet<E> extends Collections.CheckedCollection<E> implements Set<E>, Serializable
    {
        private static final long serialVersionUID = 4694047833775013803L;
        
        CheckedSet(Set<E> set, Class<E> clazz) {
            super(set, clazz);
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.c.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.c.hashCode();
        }
    }
    
    static class CheckedSortedSet<E> extends Collections.CheckedSet<E> implements SortedSet<E>, Serializable
    {
        private static final long serialVersionUID = 1599911165492914959L;
        private final SortedSet<E> ss;
        
        CheckedSortedSet(SortedSet<E> ss, Class<E> clazz) {
            super(ss, clazz);
            this.ss = ss;
        }
        
        @Override
        public Comparator<? super E> comparator() {
            return this.ss.comparator();
        }
        
        @Override
        public E first() {
            return this.ss.first();
        }
        
        @Override
        public E last() {
            return this.ss.last();
        }
        
        @Override
        public SortedSet<E> subSet(E e, E e2) {
            return Collections.checkedSortedSet(this.ss.subSet(e, e2), this.type);
        }
        
        @Override
        public SortedSet<E> headSet(E e) {
            return Collections.checkedSortedSet(this.ss.headSet(e), this.type);
        }
        
        @Override
        public SortedSet<E> tailSet(E e) {
            return Collections.checkedSortedSet(this.ss.tailSet(e), this.type);
        }
    }
    
    static class CheckedNavigableSet<E> extends Collections.CheckedSortedSet<E> implements NavigableSet<E>, Serializable
    {
        private static final long serialVersionUID = -5429120189805438922L;
        private final NavigableSet<E> ns;
        
        CheckedNavigableSet(NavigableSet<E> ns, Class<E> clazz) {
            super(ns, clazz);
            this.ns = ns;
        }
        
        @Override
        public E lower(E e) {
            return this.ns.lower(e);
        }
        
        @Override
        public E floor(E e) {
            return this.ns.floor(e);
        }
        
        @Override
        public E ceiling(E e) {
            return this.ns.ceiling(e);
        }
        
        @Override
        public E higher(E e) {
            return this.ns.higher(e);
        }
        
        @Override
        public E pollFirst() {
            return this.ns.pollFirst();
        }
        
        @Override
        public E pollLast() {
            return this.ns.pollLast();
        }
        
        @Override
        public NavigableSet<E> descendingSet() {
            return Collections.checkedNavigableSet(this.ns.descendingSet(), this.type);
        }
        
        @Override
        public Iterator<E> descendingIterator() {
            return Collections.checkedNavigableSet(this.ns.descendingSet(), this.type).iterator();
        }
        
        @Override
        public NavigableSet<E> subSet(E e, E e2) {
            return Collections.checkedNavigableSet(this.ns.subSet(e, true, e2, false), this.type);
        }
        
        @Override
        public NavigableSet<E> headSet(E e) {
            return Collections.checkedNavigableSet(this.ns.headSet(e, false), this.type);
        }
        
        @Override
        public NavigableSet<E> tailSet(E e) {
            return Collections.checkedNavigableSet(this.ns.tailSet(e, true), this.type);
        }
        
        @Override
        public NavigableSet<E> subSet(E e, boolean b, E e2, boolean b2) {
            return Collections.checkedNavigableSet(this.ns.subSet(e, b, e2, b2), this.type);
        }
        
        @Override
        public NavigableSet<E> headSet(E e, boolean b) {
            return Collections.checkedNavigableSet(this.ns.headSet(e, b), this.type);
        }
        
        @Override
        public NavigableSet<E> tailSet(E e, boolean b) {
            return Collections.checkedNavigableSet(this.ns.tailSet(e, b), this.type);
        }
    }
    
    static class CheckedList<E> extends Collections.CheckedCollection<E> implements List<E>
    {
        private static final long serialVersionUID = 65247728283967356L;
        final List<E> list;
        
        CheckedList(List<E> list, Class<E> clazz) {
            super(list, clazz);
            this.list = list;
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.list.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.list.hashCode();
        }
        
        @Override
        public E get(int n) {
            return this.list.get(n);
        }
        
        @Override
        public E remove(int n) {
            return this.list.remove(n);
        }
        
        @Override
        public int indexOf(Object o) {
            return this.list.indexOf(o);
        }
        
        @Override
        public int lastIndexOf(Object o) {
            return this.list.lastIndexOf(o);
        }
        
        @Override
        public E set(int n, E e) {
            return this.list.set(n, this.typeCheck(e));
        }
        
        @Override
        public void add(int n, E e) {
            this.list.add(n, this.typeCheck(e));
        }
        
        @Override
        public boolean addAll(int n, Collection<? extends E> collection) {
            return this.list.addAll(n, this.checkedCopyOf(collection));
        }
        
        @Override
        public ListIterator<E> listIterator() {
            return this.listIterator(0);
        }
        
        @Override
        public ListIterator<E> listIterator(int n) {
            return new ListIterator<E>() {
                
                @Override
                public boolean hasNext() {
                    return i.hasNext();
                }
                
                @Override
                public E next() {
                    return i.next();
                }
                
                @Override
                public boolean hasPrevious() {
                    return i.hasPrevious();
                }
                
                @Override
                public E previous() {
                    return i.previous();
                }
                
                @Override
                public int nextIndex() {
                    return i.nextIndex();
                }
                
                @Override
                public int previousIndex() {
                    return i.previousIndex();
                }
                
                @Override
                public void remove() {
                    i.remove();
                }
                
                @Override
                public void set(E e) {
                    i.set(Collections.CheckedList.this.typeCheck(e));
                }
                
                @Override
                public void add(E e) {
                    i.add(Collections.CheckedList.this.typeCheck(e));
                }
                
                @Override
                public void forEachRemaining(Consumer<? super E> consumer) {
                    i.forEachRemaining(consumer);
                }
            };
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            return new Collections.CheckedList(this.list.subList(n, n2), this.type);
        }
        
        @Override
        public void replaceAll(UnaryOperator<E> unaryOperator) {
            Objects.requireNonNull(unaryOperator);
            this.list.replaceAll(o/* java.lang.Object, */ -> this.typeCheck(p0.apply(o)));
        }
        
        @Override
        public void sort(Comparator<? super E> comparator) {
            this.list.sort(comparator);
        }
    }
    
    static class CheckedRandomAccessList<E> extends Collections.CheckedList<E> implements RandomAccess
    {
        private static final long serialVersionUID = 1638200125423088369L;
        
        CheckedRandomAccessList(List<E> list, Class<E> clazz) {
            super(list, clazz);
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            return new Collections.CheckedRandomAccessList(this.list.subList(n, n2), this.type);
        }
    }
    
    private static class CheckedMap<K, V> implements Map<K, V>, Serializable
    {
        private static final long serialVersionUID = 5742860141034234728L;
        private final Map<K, V> m;
        final Class<K> keyType;
        final Class<V> valueType;
        private transient Set<Map.Entry<K, V>> entrySet;
        
        private void typeCheck(Object o, Object o2) {
            if (o != null && !this.keyType.isInstance(o)) {
                throw new ClassCastException(this.badKeyMsg(o));
            }
            if (o2 != null && !this.valueType.isInstance(o2)) {
                throw new ClassCastException(this.badValueMsg(o2));
            }
            return;
        }
        
        private BiFunction<? super K, ? super V, ? extends V> typeCheck(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            Objects.requireNonNull(biFunction);
            return (o, o2)/* java.lang.Object,java.lang.Object, */ -> {
                Object o3 = p0.apply(o, o2);
                this.typeCheck(o, o3);
                return o3;
            };
        }
        
        private String badKeyMsg(Object o) {
            return "Attempt to insert " + o.getClass() + " key into map with key type " + this.keyType;
        }
        
        private String badValueMsg(Object o) {
            return "Attempt to insert " + o.getClass() + " value into map with value type " + this.valueType;
        }
        
        CheckedMap(Map<K, V> map, Class<K> clazz, Class<V> clazz2) {
            this.m = (Map<K, V>)Objects.requireNonNull(map);
            this.keyType = (Class<K>)Objects.requireNonNull(clazz);
            this.valueType = (Class<V>)Objects.requireNonNull(clazz2);
        }
        
        @Override
        public int size() {
            return this.m.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.m.isEmpty();
        }
        
        @Override
        public boolean containsKey(Object o) {
            return this.m.containsKey(o);
        }
        
        @Override
        public boolean containsValue(Object o) {
            return this.m.containsValue(o);
        }
        
        @Override
        public V get(Object o) {
            return this.m.get(o);
        }
        
        @Override
        public V remove(Object o) {
            return this.m.remove(o);
        }
        
        @Override
        public void clear() {
            this.m.clear();
        }
        
        @Override
        public Set<K> keySet() {
            return this.m.keySet();
        }
        
        @Override
        public Collection<V> values() {
            return this.m.values();
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.m.equals(o);
        }
        
        @Override
        public int hashCode() {
            return this.m.hashCode();
        }
        
        @Override
        public String toString() {
            return this.m.toString();
        }
        
        @Override
        public V put(K k, V v) {
            this.typeCheck(k, v);
            return this.m.put(k, v);
        }
        
        @Override
        public void putAll(Map<? extends K, ? extends V> map) {
            Object[] array = map.entrySet().toArray();
            ArrayList list = new ArrayList(array.length);
            for (int length = array.length, i = 0; i < length; ++i) {
                Map.Entry<Object, Object> entry = (Map.Entry)array[i];
                Object key = entry.getKey();
                Object value = entry.getValue();
                this.typeCheck(key, value);
                list.add(new AbstractMap.SimpleImmutableEntry(key, value));
            }
            for (Map.Entry entry2 : list) {
                this.m.put(entry2.getKey(), entry2.getValue());
            }
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            if (this.entrySet == null) {
                this.entrySet = new Collections.CheckedMap.CheckedEntrySet(this.m.entrySet(), this.valueType);
            }
            return this.entrySet;
        }
        
        @Override
        public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
            this.m.forEach(biConsumer);
        }
        
        @Override
        public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            this.m.replaceAll(this.typeCheck(biFunction));
        }
        
        @Override
        public V putIfAbsent(K k, V v) {
            this.typeCheck(k, v);
            return this.m.putIfAbsent(k, v);
        }
        
        @Override
        public boolean remove(Object o, Object o2) {
            return this.m.remove(o, o2);
        }
        
        @Override
        public boolean replace(K k, V v, V v2) {
            this.typeCheck(k, v2);
            return this.m.replace(k, v, v2);
        }
        
        @Override
        public V replace(K k, V v) {
            this.typeCheck(k, v);
            return this.m.replace(k, v);
        }
        
        @Override
        public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
            Objects.requireNonNull(function);
            return this.m.computeIfAbsent(k, o/* java.lang.Object, */ -> {
                Object o2 = p1.apply(o);
                this.typeCheck(o, o2);
                return o2;
            });
        }
        
        @Override
        public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            return this.m.computeIfPresent(k, this.typeCheck(biFunction));
        }
        
        @Override
        public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            return this.m.compute(k, this.typeCheck(biFunction));
        }
        
        @Override
        public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
            Objects.requireNonNull(biFunction);
            return this.m.merge(k, v, (o, o2)/* java.lang.Object,java.lang.Object, */ -> {
                Object o3 = p2.apply(o, o2);
                this.typeCheck(null, o3);
                return o3;
            });
        }
        
        static class CheckedEntrySet<K, V> implements Set<Map.Entry<K, V>>
        {
            private final Set<Map.Entry<K, V>> s;
            private final Class<V> valueType;
            
            CheckedEntrySet(Set<Map.Entry<K, V>> s, Class<V> valueType) {
                this.s = s;
                this.valueType = valueType;
            }
            
            @Override
            public int size() {
                return this.s.size();
            }
            
            @Override
            public boolean isEmpty() {
                return this.s.isEmpty();
            }
            
            @Override
            public String toString() {
                return this.s.toString();
            }
            
            @Override
            public int hashCode() {
                return this.s.hashCode();
            }
            
            @Override
            public void clear() {
                this.s.clear();
            }
            
            @Override
            public boolean add(Map.Entry<K, V> entry) {
                throw new UnsupportedOperationException();
            }
            
            @Override
            public boolean addAll(Collection<? extends Map.Entry<K, V>> collection) {
                throw new UnsupportedOperationException();
            }
            
            @Override
            public Iterator<Map.Entry<K, V>> iterator() {
                return new Iterator<Map.Entry<K, V>>() {
                    
                    @Override
                    public boolean hasNext() {
                        return i.hasNext();
                    }
                    
                    @Override
                    public void remove() {
                        i.remove();
                    }
                    
                    @Override
                    public Map.Entry<K, V> next() {
                        return Collections.CheckedMap.CheckedEntrySet.checkedEntry((Map.Entry)i.next(), valueType);
                    }
                };
            }
            
            @Override
            public Object[] toArray() {
                Object[] array = this.s.toArray();
                Object[] array2 = Collections.CheckedMap.CheckedEntrySet.CheckedEntry.class.isInstance(array.getClass().getComponentType()) ? array : new Object[array.length];
                for (int i = 0; i < array.length; ++i) {
                    array2[i] = checkedEntry((Map.Entry)array[i], this.valueType);
                }
                return array2;
            }
            
            @Override
            public <T> T[] toArray(T[] array) {
                Object[] array2 = this.s.toArray((array.length == 0) ? array : Arrays.copyOf(array, 0));
                for (int i = 0; i < array2.length; ++i) {
                    array2[i] = checkedEntry((Map.Entry)array2[i], this.valueType);
                }
                if (array2.length > array.length) {
                    return array2;
                }
                System.arraycopy(array2, 0, array, 0, array2.length);
                if (array.length > array2.length) {
                    array[array2.length] = null;
                    return array;
                }
                return array;
            }
            
            @Override
            public boolean contains(Object o) {
                if (!(o instanceof Map.Entry)) {
                    return false;
                }
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                return this.s.contains((entry instanceof Collections.CheckedMap.CheckedEntrySet.CheckedEntry) ? entry : checkedEntry(entry, this.valueType));
            }
            
            @Override
            public boolean containsAll(Collection<?> collection) {
                Iterator<Object> iterator = collection.iterator();
                while (iterator.hasNext()) {
                    if (!this.contains(iterator.next())) {
                        return false;
                    }
                    continue;
                }
                return true;
            }
            
            @Override
            public boolean remove(Object o) {
                return o instanceof Map.Entry && this.s.remove(new AbstractMap.SimpleImmutableEntry((Map.Entry)o));
            }
            
            @Override
            public boolean removeAll(Collection<?> collection) {
                return this.batchRemove(collection, false);
            }
            
            @Override
            public boolean retainAll(Collection<?> collection) {
                return this.batchRemove(collection, true);
            }
            
            private boolean batchRemove(Collection<?> collection, boolean b) {
                Objects.requireNonNull(collection);
                boolean n = false;
                Iterator<Map.Entry<K, V>> iterator = this.iterator();
                while (iterator.hasNext()) {
                    if (collection.contains(iterator.next()) == b) {
                        continue;
                    }
                    iterator.remove();
                    n = true;
                }
                return n;
            }
            
            @Override
            public boolean equals(Object o) {
                if (o == this) {
                    return true;
                }
                if (!(o instanceof Set)) {
                    return false;
                }
                Set<Object> set = (Set)o;
                return set.size() == this.s.size() && this.containsAll(set);
            }
            
            static <K, V, T> Collections.CheckedMap.CheckedEntrySet.CheckedEntry<K, V, T> checkedEntry(Map.Entry<K, V> entry, Class<T> clazz) {
                return new Collections.CheckedMap.CheckedEntrySet.CheckedEntry(entry, clazz);
            }
            
            private static class CheckedEntry<K, V, T> implements Map.Entry<K, V>
            {
                private final Map.Entry<K, V> e;
                private final Class<T> valueType;
                
                CheckedEntry(Map.Entry<K, V> entry, Class<T> clazz) {
                    this.e = (Map.Entry<K, V>)Objects.requireNonNull(entry);
                    this.valueType = (Class<T>)Objects.requireNonNull(clazz);
                }
                
                @Override
                public K getKey() {
                    return this.e.getKey();
                }
                
                @Override
                public V getValue() {
                    return this.e.getValue();
                }
                
                @Override
                public int hashCode() {
                    return this.e.hashCode();
                }
                
                @Override
                public String toString() {
                    return this.e.toString();
                }
                
                @Override
                public V setValue(V value) {
                    if (value != null && !this.valueType.isInstance(value)) {
                        throw new ClassCastException(this.badValueMsg(value));
                    }
                    return this.e.setValue(value);
                }
                
                private String badValueMsg(Object o) {
                    return "Attempt to insert " + o.getClass() + " value into map with value type " + this.valueType;
                }
                
                @Override
                public boolean equals(Object o) {
                    return o == this || (o instanceof Map.Entry && this.e.equals(new AbstractMap.SimpleImmutableEntry((Map.Entry)o)));
                }
            }
        }
    }
    
    static class CheckedSortedMap<K, V> extends Collections.CheckedMap<K, V> implements SortedMap<K, V>, Serializable
    {
        private static final long serialVersionUID = 1599671320688067438L;
        private final SortedMap<K, V> sm;
        
        CheckedSortedMap(SortedMap<K, V> sm, Class<K> clazz, Class<V> clazz2) {
            super(sm, clazz, clazz2);
            this.sm = sm;
        }
        
        @Override
        public Comparator<? super K> comparator() {
            return this.sm.comparator();
        }
        
        @Override
        public K firstKey() {
            return this.sm.firstKey();
        }
        
        @Override
        public K lastKey() {
            return this.sm.lastKey();
        }
        
        @Override
        public SortedMap<K, V> subMap(K k, K i) {
            return Collections.checkedSortedMap(this.sm.subMap(k, i), this.keyType, this.valueType);
        }
        
        @Override
        public SortedMap<K, V> headMap(K k) {
            return Collections.checkedSortedMap(this.sm.headMap(k), this.keyType, this.valueType);
        }
        
        @Override
        public SortedMap<K, V> tailMap(K k) {
            return Collections.checkedSortedMap(this.sm.tailMap(k), this.keyType, this.valueType);
        }
    }
    
    static class CheckedNavigableMap<K, V> extends Collections.CheckedSortedMap<K, V> implements NavigableMap<K, V>, Serializable
    {
        private static final long serialVersionUID = -4852462692372534096L;
        private final NavigableMap<K, V> nm;
        
        CheckedNavigableMap(NavigableMap<K, V> nm, Class<K> clazz, Class<V> clazz2) {
            super(nm, clazz, clazz2);
            this.nm = nm;
        }
        
        @Override
        public Comparator<? super K> comparator() {
            return this.nm.comparator();
        }
        
        @Override
        public K firstKey() {
            return this.nm.firstKey();
        }
        
        @Override
        public K lastKey() {
            return this.nm.lastKey();
        }
        
        @Override
        public Map.Entry<K, V> lowerEntry(K k) {
            Map.Entry<K, V> lowerEntry = this.nm.lowerEntry(k);
            return (null != lowerEntry) ? new CheckedEntrySet.CheckedEntry(lowerEntry, this.valueType) : null;
        }
        
        @Override
        public K lowerKey(K k) {
            return this.nm.lowerKey(k);
        }
        
        @Override
        public Map.Entry<K, V> floorEntry(K k) {
            Map.Entry<K, V> floorEntry = this.nm.floorEntry(k);
            return (null != floorEntry) ? new CheckedEntrySet.CheckedEntry(floorEntry, this.valueType) : null;
        }
        
        @Override
        public K floorKey(K k) {
            return this.nm.floorKey(k);
        }
        
        @Override
        public Map.Entry<K, V> ceilingEntry(K k) {
            Map.Entry<K, V> ceilingEntry = this.nm.ceilingEntry(k);
            return (null != ceilingEntry) ? new CheckedEntrySet.CheckedEntry(ceilingEntry, this.valueType) : null;
        }
        
        @Override
        public K ceilingKey(K k) {
            return this.nm.ceilingKey(k);
        }
        
        @Override
        public Map.Entry<K, V> higherEntry(K k) {
            Map.Entry<K, V> higherEntry = this.nm.higherEntry(k);
            return (null != higherEntry) ? new CheckedEntrySet.CheckedEntry(higherEntry, this.valueType) : null;
        }
        
        @Override
        public K higherKey(K k) {
            return this.nm.higherKey(k);
        }
        
        @Override
        public Map.Entry<K, V> firstEntry() {
            Map.Entry<K, V> firstEntry = this.nm.firstEntry();
            return (null != firstEntry) ? new CheckedEntrySet.CheckedEntry(firstEntry, this.valueType) : null;
        }
        
        @Override
        public Map.Entry<K, V> lastEntry() {
            Map.Entry<K, V> lastEntry = this.nm.lastEntry();
            return (null != lastEntry) ? new CheckedEntrySet.CheckedEntry(lastEntry, this.valueType) : null;
        }
        
        @Override
        public Map.Entry<K, V> pollFirstEntry() {
            Map.Entry<K, V> pollFirstEntry = this.nm.pollFirstEntry();
            return (null == pollFirstEntry) ? null : new CheckedEntrySet.CheckedEntry(pollFirstEntry, this.valueType);
        }
        
        @Override
        public Map.Entry<K, V> pollLastEntry() {
            Map.Entry<K, V> pollLastEntry = this.nm.pollLastEntry();
            return (null == pollLastEntry) ? null : new CheckedEntrySet.CheckedEntry(pollLastEntry, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> descendingMap() {
            return Collections.checkedNavigableMap(this.nm.descendingMap(), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableSet<K> keySet() {
            return this.navigableKeySet();
        }
        
        @Override
        public NavigableSet<K> navigableKeySet() {
            return Collections.checkedNavigableSet(this.nm.navigableKeySet(), this.keyType);
        }
        
        @Override
        public NavigableSet<K> descendingKeySet() {
            return Collections.checkedNavigableSet(this.nm.descendingKeySet(), this.keyType);
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, K i) {
            return Collections.checkedNavigableMap(this.nm.subMap(k, true, i, false), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k) {
            return Collections.checkedNavigableMap(this.nm.headMap(k, false), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k) {
            return Collections.checkedNavigableMap(this.nm.tailMap(k, true), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> subMap(K k, boolean b, K i, boolean b2) {
            return Collections.checkedNavigableMap(this.nm.subMap(k, b, i, b2), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> headMap(K k, boolean b) {
            return Collections.checkedNavigableMap(this.nm.headMap(k, b), this.keyType, this.valueType);
        }
        
        @Override
        public NavigableMap<K, V> tailMap(K k, boolean b) {
            return Collections.checkedNavigableMap(this.nm.tailMap(k, b), this.keyType, this.valueType);
        }
    }
    
    private static class EmptyIterator<E> implements Iterator<E>
    {
        static final Collections.EmptyIterator<Object> EMPTY_ITERATOR;
        
        @Override
        public boolean hasNext() {
            return false;
        }
        
        @Override
        public E next() {
            throw new NoSuchElementException();
        }
        
        @Override
        public void remove() {
            throw new IllegalStateException();
        }
        
        @Override
        public void forEachRemaining(Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
        }
        
        static {
            EMPTY_ITERATOR = new Collections.EmptyIterator();
        }
    }
    
    private static class EmptyListIterator<E> extends Collections.EmptyIterator<E> implements ListIterator<E>
    {
        static final Collections.EmptyListIterator<Object> EMPTY_ITERATOR;
        
        @Override
        public boolean hasPrevious() {
            return false;
        }
        
        @Override
        public E previous() {
            throw new NoSuchElementException();
        }
        
        @Override
        public int nextIndex() {
            return 0;
        }
        
        @Override
        public int previousIndex() {
            return -1;
        }
        
        @Override
        public void set(E e) {
            throw new IllegalStateException();
        }
        
        @Override
        public void add(E e) {
            throw new UnsupportedOperationException();
        }
        
        static {
            EMPTY_ITERATOR = new Collections.EmptyListIterator();
        }
    }
    
    private static class EmptyEnumeration<E> implements Enumeration<E>
    {
        static final Collections.EmptyEnumeration<Object> EMPTY_ENUMERATION;
        
        @Override
        public boolean hasMoreElements() {
            return false;
        }
        
        @Override
        public E nextElement() {
            throw new NoSuchElementException();
        }
        
        static {
            EMPTY_ENUMERATION = new Collections.EmptyEnumeration();
        }
    }
    
    private static class EmptySet<E> extends AbstractSet<E> implements Serializable
    {
        private static final long serialVersionUID = 1582296315990362920L;
        
        @Override
        public Iterator<E> iterator() {
            return Collections.emptyIterator();
        }
        
        @Override
        public int size() {
            return 0;
        }
        
        @Override
        public boolean isEmpty() {
            return true;
        }
        
        @Override
        public boolean contains(Object o) {
            return false;
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return collection.isEmpty();
        }
        
        @Override
        public Object[] toArray() {
            return new Object[0];
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            if (array.length > 0) {
                array[0] = null;
                return array;
            }
            return array;
        }
        
        @Override
        public void forEach(Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            Objects.requireNonNull(predicate);
            return false;
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return Spliterators.emptySpliterator();
        }
        
        private Object readResolve() {
            return Collections.EMPTY_SET;
        }
    }
    
    private static class EmptyList<E> extends AbstractList<E> implements RandomAccess, Serializable
    {
        private static final long serialVersionUID = 8842843931221139166L;
        
        @Override
        public Iterator<E> iterator() {
            return Collections.emptyIterator();
        }
        
        @Override
        public ListIterator<E> listIterator() {
            return Collections.emptyListIterator();
        }
        
        @Override
        public int size() {
            return 0;
        }
        
        @Override
        public boolean isEmpty() {
            return true;
        }
        
        @Override
        public boolean contains(Object o) {
            return false;
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return collection.isEmpty();
        }
        
        @Override
        public Object[] toArray() {
            return new Object[0];
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            if (array.length > 0) {
                array[0] = null;
                return array;
            }
            return array;
        }
        
        @Override
        public E get(int i) {
            throw new IndexOutOfBoundsException("Index: " + i);
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof List && ((List)o).isEmpty();
        }
        
        @Override
        public int hashCode() {
            return 1;
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            Objects.requireNonNull(predicate);
            return false;
        }
        
        @Override
        public void replaceAll(UnaryOperator<E> unaryOperator) {
            Objects.requireNonNull(unaryOperator);
        }
        
        @Override
        public void sort(Comparator<? super E> comparator) {
        }
        
        @Override
        public void forEach(Consumer<? super E> consumer) {
            Objects.requireNonNull(consumer);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return Spliterators.emptySpliterator();
        }
        
        private Object readResolve() {
            return Collections.EMPTY_LIST;
        }
    }
    
    private static class EmptyMap<K, V> extends AbstractMap<K, V> implements Serializable
    {
        private static final long serialVersionUID = 6428348081105594320L;
        
        @Override
        public int size() {
            return 0;
        }
        
        @Override
        public boolean isEmpty() {
            return true;
        }
        
        @Override
        public boolean containsKey(Object o) {
            return false;
        }
        
        @Override
        public boolean containsValue(Object o) {
            return false;
        }
        
        @Override
        public V get(Object o) {
            return null;
        }
        
        @Override
        public Set<K> keySet() {
            return Collections.emptySet();
        }
        
        @Override
        public Collection<V> values() {
            return Collections.emptySet();
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            return Collections.emptySet();
        }
        
        @Override
        public boolean equals(Object o) {
            return o instanceof Map && ((Map)o).isEmpty();
        }
        
        @Override
        public int hashCode() {
            return 0;
        }
        
        @Override
        public V getOrDefault(Object o, V v) {
            return v;
        }
        
        @Override
        public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
            Objects.requireNonNull(biConsumer);
        }
        
        @Override
        public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            Objects.requireNonNull(biFunction);
        }
        
        @Override
        public V putIfAbsent(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean remove(Object o, Object o2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean replace(K k, V v, V v2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V replace(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        private Object readResolve() {
            return Collections.EMPTY_MAP;
        }
    }
    
    private static class SingletonSet<E> extends AbstractSet<E> implements Serializable
    {
        private static final long serialVersionUID = 3193687207550431679L;
        private final E element;
        
        SingletonSet(E element) {
            this.element = element;
        }
        
        @Override
        public Iterator<E> iterator() {
            return Collections.singletonIterator(this.element);
        }
        
        @Override
        public int size() {
            return 1;
        }
        
        @Override
        public boolean contains(Object o) {
            return Collections.eq(o, this.element);
        }
        
        @Override
        public void forEach(Consumer<? super E> consumer) {
            consumer.accept(this.element);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return Collections.singletonSpliterator(this.element);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            throw new UnsupportedOperationException();
        }
    }
    
    private static class SingletonList<E> extends AbstractList<E> implements RandomAccess, Serializable
    {
        private static final long serialVersionUID = 3093736618740652951L;
        private final E element;
        
        SingletonList(E element) {
            this.element = element;
        }
        
        @Override
        public Iterator<E> iterator() {
            return Collections.singletonIterator(this.element);
        }
        
        @Override
        public int size() {
            return 1;
        }
        
        @Override
        public boolean contains(Object o) {
            return Collections.eq(o, this.element);
        }
        
        @Override
        public E get(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("Index: " + i + ", Size: 1");
            }
            return this.element;
        }
        
        @Override
        public void forEach(Consumer<? super E> consumer) {
            consumer.accept(this.element);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void replaceAll(UnaryOperator<E> unaryOperator) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void sort(Comparator<? super E> comparator) {
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return Collections.singletonSpliterator(this.element);
        }
    }
    
    private static class SingletonMap<K, V> extends AbstractMap<K, V> implements Serializable
    {
        private static final long serialVersionUID = -6979724477215052911L;
        private final K k;
        private final V v;
        private transient Set<K> keySet;
        private transient Set<Map.Entry<K, V>> entrySet;
        private transient Collection<V> values;
        
        SingletonMap(K k, V v) {
            this.k = k;
            this.v = v;
        }
        
        @Override
        public int size() {
            return 1;
        }
        
        @Override
        public boolean isEmpty() {
            return false;
        }
        
        @Override
        public boolean containsKey(Object o) {
            return Collections.eq(o, this.k);
        }
        
        @Override
        public boolean containsValue(Object o) {
            return Collections.eq(o, this.v);
        }
        
        @Override
        public V get(Object o) {
            return Collections.eq(o, this.k) ? this.v : null;
        }
        
        @Override
        public Set<K> keySet() {
            if (this.keySet == null) {
                this.keySet = Collections.singleton(this.k);
            }
            return this.keySet;
        }
        
        @Override
        public Set<Map.Entry<K, V>> entrySet() {
            if (this.entrySet == null) {
                this.entrySet = Collections.singleton(new AbstractMap.SimpleImmutableEntry(this.k, this.v));
            }
            return this.entrySet;
        }
        
        @Override
        public Collection<V> values() {
            if (this.values == null) {
                this.values = Collections.singleton(this.v);
            }
            return this.values;
        }
        
        @Override
        public V getOrDefault(Object o, V v) {
            return Collections.eq(o, this.k) ? this.v : v;
        }
        
        @Override
        public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
            biConsumer.accept(this.k, this.v);
        }
        
        @Override
        public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V putIfAbsent(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean remove(Object o, Object o2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public boolean replace(K k, V v, V v2) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V replace(K k, V v) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
            throw new UnsupportedOperationException();
        }
    }
    
    private static class CopiesList<E> extends AbstractList<E> implements RandomAccess, Serializable
    {
        private static final long serialVersionUID = 2739099268398711800L;
        final int n;
        final E element;
        
        CopiesList(int n, E element) {
            assert n >= 0;
            this.n = n;
            this.element = element;
            return;
        }
        
        @Override
        public int size() {
            return this.n;
        }
        
        @Override
        public boolean contains(Object o) {
            return this.n != 0 && Collections.eq(o, this.element);
        }
        
        @Override
        public int indexOf(Object o) {
            return this.contains(o) ? 0 : -1;
        }
        
        @Override
        public int lastIndexOf(Object o) {
            return this.contains(o) ? (this.n - 1) : -1;
        }
        
        @Override
        public E get(int i) {
            if (i < 0 || i >= this.n) {
                throw new IndexOutOfBoundsException("Index: " + i + ", Size: " + this.n);
            }
            return this.element;
        }
        
        @Override
        public Object[] toArray() {
            Object[] array = new Object[this.n];
            if (this.element != null) {
                Arrays.fill(array, 0, this.n, this.element);
                return array;
            }
            return array;
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            int n = this.n;
            if (array.length < n) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), n);
                if (this.element != null) {
                    Arrays.fill(array, 0, n, this.element);
                }
            }
            else {
                Arrays.fill(array, 0, n, this.element);
                if (array.length > n) {
                    array[n] = null;
                }
            }
            return array;
        }
        
        @Override
        public List<E> subList(int n, int n2) {
            if (n < 0) {
                throw new IndexOutOfBoundsException("fromIndex = " + n);
            }
            if (n2 > this.n) {
                throw new IndexOutOfBoundsException("toIndex = " + n2);
            }
            if (n > n2) {
                throw new IllegalArgumentException("fromIndex(" + n + ") > toIndex(" + n2 + ")");
            }
            return new Collections.CopiesList(n2 - n, this.element);
        }
        
        @Override
        public int hashCode() {
            if (this.n == 0) {
                return 1;
            }
            int n = 31;
            int n2 = 1;
            for (int i = Integer.numberOfLeadingZeros(this.n) + 1; i < 32; ++i) {
                n2 *= n + 1;
                n *= n;
                if (this.n << i < 0) {
                    n *= 31;
                    n2 = n2 * 31 + 1;
                }
            }
            return n + n2 * ((this.element == null) ? 0 : this.element.hashCode());
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if ((o instanceof Collections.CopiesList)) {
                Collections.CopiesList<Object> list = (Collections.CopiesList)o;
                return this.n == list.n && (this.n == 0 || Collections.eq(this.element, list.element));
            }
            if (!(o instanceof List)) {
                return false;
            }
            int n = this.n;
            Object element = this.element;
            Iterator<Object> iterator = ((List)o).iterator();
            if (element == null) {
                while (iterator.hasNext() && n-- > 0) {
                    if (iterator.next() == null) {
                        continue;
                    }
                    return false;
                }
            }
            else {
                while (iterator.hasNext() && n-- > 0) {
                    if (element.equals(iterator.next())) {
                        continue;
                    }
                    return false;
                }
            }
            return n == 0 && !iterator.hasNext();
        }
        
        @Override
        public Stream<E> stream() {
            return IntStream.range(0, this.n).mapToObj(p0/* int, */ -> this.element);
        }
        
        @Override
        public Stream<E> parallelStream() {
            return IntStream.range(0, this.n).parallel().mapToObj(p0/* int, */ -> this.element);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.stream().spliterator();
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
            objectInputStream.defaultReadObject();
            SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Object[].class, this.n);
        }
    }
    
    private static class ReverseComparator implements Comparator<Comparable<Object>>, Serializable
    {
        private static final long serialVersionUID = 7207038068494060240L;
        static final Collections.ReverseComparator REVERSE_ORDER;
        
        @Override
        public int compare(Comparable<Object> comparable, Comparable<Object> comparable2) {
            return comparable2.compareTo(comparable);
        }
        
        private Object readResolve() {
            return Collections.reverseOrder();
        }
        
        @Override
        public Comparator<Comparable<Object>> reversed() {
            return Comparator.naturalOrder();
        }
        
        static {
            REVERSE_ORDER = new Collections.ReverseComparator();
        }
    }
    
    private static class ReverseComparator2<T> implements Comparator<T>, Serializable
    {
        private static final long serialVersionUID = 4374092139857L;
        final Comparator<T> cmp;
        
        ReverseComparator2(Comparator<T> cmp) {
            assert cmp != null;
            this.cmp = cmp;
            return;
        }
        
        @Override
        public int compare(T t, T t2) {
            return this.cmp.compare(t2, t);
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || (o instanceof Collections.ReverseComparator2 && this.cmp.equals(((Collections.ReverseComparator2)o).cmp));
        }
        
        @Override
        public int hashCode() {
            return this.cmp.hashCode() ^ Integer.MIN_VALUE;
        }
        
        @Override
        public Comparator<T> reversed() {
            return this.cmp;
        }
    }
    
    private static class SetFromMap<E> extends AbstractSet<E> implements Set<E>, Serializable
    {
        private final Map<E, Boolean> m;
        private transient Set<E> s;
        private static final long serialVersionUID = 2454657854757543876L;
        
        SetFromMap(Map<E, Boolean> m) {
            if (!m.isEmpty()) {
                throw new IllegalArgumentException("Map is non-empty");
            }
            this.m = m;
            this.s = m.keySet();
            return;
        }
        
        @Override
        public void clear() {
            this.m.clear();
        }
        
        @Override
        public int size() {
            return this.m.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.m.isEmpty();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.m.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            return this.m.remove(o) != null;
        }
        
        @Override
        public boolean add(E e) {
            return this.m.put(e, Boolean.TRUE) == null;
        }
        
        @Override
        public Iterator<E> iterator() {
            return this.s.iterator();
        }
        
        @Override
        public Object[] toArray() {
            return this.s.toArray();
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            return this.s.toArray(array);
        }
        
        @Override
        public String toString() {
            return this.s.toString();
        }
        
        @Override
        public int hashCode() {
            return this.s.hashCode();
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || this.s.equals(o);
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return this.s.containsAll(collection);
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            return this.s.removeAll(collection);
        }
        
        @Override
        public boolean retainAll(Collection<?> collection) {
            return this.s.retainAll(collection);
        }
        
        @Override
        public void forEach(Consumer<? super E> action) {
            this.s.forEach(action);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            return this.s.removeIf(predicate);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.s.spliterator();
        }
        
        @Override
        public Stream<E> stream() {
            return this.s.stream();
        }
        
        @Override
        public Stream<E> parallelStream() {
            return this.s.parallelStream();
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
            objectInputStream.defaultReadObject();
            this.s = this.m.keySet();
        }
    }
    
    static class AsLIFOQueue<E> extends AbstractQueue<E> implements Queue<E>, Serializable
    {
        private static final long serialVersionUID = 1802017725587941708L;
        private final Deque<E> q;
        
        AsLIFOQueue(Deque<E> q) {
            this.q = q;
        }
        
        @Override
        public boolean add(E e) {
            this.q.addFirst(e);
            return true;
        }
        
        @Override
        public boolean offer(E e) {
            return this.q.offerFirst(e);
        }
        
        @Override
        public E poll() {
            return this.q.pollFirst();
        }
        
        @Override
        public E remove() {
            return this.q.removeFirst();
        }
        
        @Override
        public E peek() {
            return this.q.peekFirst();
        }
        
        @Override
        public E element() {
            return this.q.getFirst();
        }
        
        @Override
        public void clear() {
            this.q.clear();
        }
        
        @Override
        public int size() {
            return this.q.size();
        }
        
        @Override
        public boolean isEmpty() {
            return this.q.isEmpty();
        }
        
        @Override
        public boolean contains(Object o) {
            return this.q.contains(o);
        }
        
        @Override
        public boolean remove(Object o) {
            return this.q.remove(o);
        }
        
        @Override
        public Iterator<E> iterator() {
            return this.q.iterator();
        }
        
        @Override
        public Object[] toArray() {
            return this.q.toArray();
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            return this.q.toArray(array);
        }
        
        @Override
        public String toString() {
            return this.q.toString();
        }
        
        @Override
        public boolean containsAll(Collection<?> collection) {
            return this.q.containsAll(collection);
        }
        
        @Override
        public boolean removeAll(Collection<?> collection) {
            return this.q.removeAll(collection);
        }
        
        @Override
        public boolean retainAll(Collection<?> collection) {
            return this.q.retainAll(collection);
        }
        
        @Override
        public void forEach(Consumer<? super E> action) {
            this.q.forEach(action);
        }
        
        @Override
        public boolean removeIf(Predicate<? super E> predicate) {
            return this.q.removeIf(predicate);
        }
        
        @Override
        public Spliterator<E> spliterator() {
            return this.q.spliterator();
        }
        
        @Override
        public Stream<E> stream() {
            return this.q.stream();
        }
        
        @Override
        public Stream<E> parallelStream() {
            return this.q.parallelStream();
        }
    }
}
